#define _HUYGENS_INT_
#define _CRT_SECURE_NO_WARNINGS
#define _CRT_NON_CONFORMING_SWPRINTFS

#include "stdafx.h"
#include "resource.h"
#include "filedlg.h"
#include <csrimage.h>
#include <csrfile.h>
#include <Windowsx.h>
#include <CommCtrl.h>
#include <map>

using namespace csrutils;

#define STATUS_UPD_TIMER	0x01

const wchar_t *ChainView_ClassName = L"CLS_HCHAINVIEW";
bool ChainView_RegClass_State = false;
std::map<HWND,HChainView*> ChainView_Wnds;

void ChainView_RegClass(HINSTANCE hInst);
LRESULT CALLBACK ChainView_WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

const FILDEDLG_FILTERSPEC ChainView_FileTypes[] = {
    { L"Optical chain (*.chain)",   L"*.chain" },
	{ 0, 0 }
};

struct HChainView_Ctx {
	HWND hWnd;
	double iview_Z;
	HLightMap *iview_map;
	HImageView *iview;
	HINSTANCE hInst;
	WCHAR szBuffer[512];
	int lastChainItem;
	HWND hWndListView;
	HWND hStatusBar;
	const HOpticalElementClass** optClasses;
	std::map<HWND,HOpticalElement*> configElements;
	ThreadMonitor monitor;
};

void ChainView_RegClass(HINSTANCE hInst) {
	if(ChainView_RegClass_State)
		return;
	ChainView_RegClass_State = true;
	WNDCLASSEX wcex;
    wcex.cbSize = sizeof(WNDCLASSEX);
    wcex.style          = CS_HREDRAW | CS_VREDRAW;
    wcex.lpfnWndProc    = ChainView_WndProc;
    wcex.cbClsExtra     = 0;
	wcex.cbWndExtra     = 0;
    wcex.hInstance      = hInst;
    wcex.hIcon          = LoadIcon(hInst, MAKEINTRESOURCE(IDI_APPICON));
    wcex.hCursor        = LoadCursor(NULL, IDC_ARROW);
    wcex.hbrBackground  = (HBRUSH)(COLOR_WINDOW+1);
    wcex.lpszMenuName   = MAKEINTRESOURCE(IDM_CHAINVIEW_MENU);
    wcex.lpszClassName  = ChainView_ClassName;
    wcex.hIconSm        = LoadIcon(wcex.hInstance, MAKEINTRESOURCE(IDI_APPLICATION));
    if ( !RegisterClassEx(&wcex) ) {
        return;
    }
}

LRESULT CALLBACK ChainView_WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	HChainView* view = ChainView_Wnds[hWnd];
	switch(message) {
	case WM_CREATE: {
		CREATESTRUCT* crt = (CREATESTRUCT*) lParam;
		view = (HChainView*) crt->lpCreateParams;
		ChainView_Wnds[hWnd] = view;
		return view->_wndproc(hWnd,message,wParam,lParam);
	}
	default:
		return view->_wndproc(hWnd,message,wParam,lParam);
	}
}

LRESULT CALLBACK ChainView_InsertDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	HChainView* view = ChainView_Wnds[hWnd];
	switch(message) {
	case WM_INITDIALOG: {
		view = (HChainView*) lParam;
		ChainView_Wnds[hWnd] = view;
		return view->_insertdlgproc(hWnd,message,wParam,lParam);
	}
	default:
		if( view==NULL )
			return 0;
		return view->_insertdlgproc(hWnd,message,wParam,lParam);
	}
}

LRESULT CALLBACK ChainView_ElMoveDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	HChainView* view = ChainView_Wnds[hWnd];
	switch(message) {
	case WM_INITDIALOG: {
		view = (HChainView*) lParam;
		ChainView_Wnds[hWnd] = view;
		return view->_elmovedlgproc(hWnd,message,wParam,lParam);
	}
	default:
		if( view==NULL )
			return 0;
		return view->_elmovedlgproc(hWnd,message,wParam,lParam);
	}
}

LRESULT CALLBACK ChainView_ElConfigDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	HChainView* view = ChainView_Wnds[hWnd];
	switch(message) {
	case WM_INITDIALOG: {
		view = (HChainView*) lParam;
		ChainView_Wnds[hWnd] = view;
		return view->_elconfdlgproc(hWnd,message,wParam,lParam);
	}
	default:
		if( view==NULL )
			return 0;
		return view->_elconfdlgproc(hWnd,message,wParam,lParam);
	}
}

LRESULT CALLBACK ChainView_SettingsDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	HChainView* view = ChainView_Wnds[hWnd];
	switch(message) {
	case WM_INITDIALOG: {
		view = (HChainView*) lParam;
		ChainView_Wnds[hWnd] = view;
		return view->_settingsdlgproc(hWnd,message,wParam,lParam);
	}
	default:
		if( view==NULL )
			return 0;
		return view->_settingsdlgproc(hWnd,message,wParam,lParam);
	}
}

HChainView::HChainView(HInstance *_inst, HOpticalChain *_chain)
		: inst(_inst), chain(_chain) {
	ctx = new HChainView_Ctx;
	ChainView_RegClass(inst->getHInst());
	ctx->hInst = inst->getHInst();
	CreateWindow(ChainView_ClassName,L"ChainView",WS_OVERLAPPEDWINDOW,
			0,0,600,600,0,0,inst->getHInst(),this);
	ShowWindow(ctx->hWnd,SW_NORMAL);
    UpdateWindow(ctx->hWnd);

	//_chainopen(true);	// DEBUG:

	/*ctx->iview_map = new HLightMap(512,512);
	ctx->iview_Z = 10.0;
	chain->render(*ctx->iview_map,ctx->iview_Z);
	ctx->iview = new HImageView(inst,ctx->iview_map,ctx->hWnd);*/
}

HChainView::~HChainView() {
	DestroyWindow(ctx->hWnd);
	delete ctx;
}

void HChainView::render() {
	ctx->monitor.abortchilds();
	ctx->monitor.setjob(0.0);
	chain->render(ctx->monitor);
}

LRESULT HChainView::_wndproc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	switch(message) {
	case WM_CREATE: {
		ctx->hWnd = hWnd;
		_initListView();
		ctx->hStatusBar = CreateWindow( STATUSCLASSNAME, NULL, SBARS_SIZEGRIP|WS_CHILD|WS_VISIBLE,
			0, 0, 0, 0, hWnd, NULL, inst->getHInst(), 0 );
		SetTimer(hWnd,STATUS_UPD_TIMER,100,0);
		//SetMenu(hWnd,LoadMenu(ctx->hInst,MEKAINTR
		break; }
	case WM_SIZE:
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	case WM_NOTIFY:
		_wndproc_notify(hWnd,message,wParam,lParam);
		break;
	case WM_COMMAND:
		switch( LOWORD(wParam) ) {
		case IDC_CHAINVIEW_EDITITEM:
			if( ctx->lastChainItem!=-1 ) {
				HOpticalElement* elem = chain->getElement(ctx->lastChainItem);
				elem->configWnd(inst,hWnd);
			}
			break;
		case IDC_CHAINVIEW_DELITEM:
			if( ctx->lastChainItem!=-1 ) {
				chain->removeElement(ctx->lastChainItem);
				ListView_DeleteItem(ctx->hWndListView,ctx->lastChainItem);
			}
			break;
		case IDC_CHAINVIEW_MOVEITEM: {
			HWND hDlg = CreateDialogParam( inst->getHInst(), MAKEINTRESOURCE(IDD_ELEMMOVE), 
				hWnd, ChainView_ElMoveDlgProc, (LPARAM)this);
			ShowWindow(hDlg,SW_NORMAL);
			break; }
		case IDC_CHAINVIEW_INSERTITEM: {
			HWND hDlg = CreateDialogParam( inst->getHInst(), MAKEINTRESOURCE(IDD_CHAINVIEW_NEWITEM), 
				hWnd, ChainView_InsertDlgProc, (LPARAM)this);
			ShowWindow(hDlg,SW_NORMAL);
			break; }
		case IDC_CHAINVIEW_CONFIG: {
			HWND hDlg = CreateDialogParam( inst->getHInst(), MAKEINTRESOURCE(IDD_ELEMCONFIG), 
				hWnd, ChainView_ElConfigDlgProc, (LPARAM)this);
			ShowWindow(hDlg,SW_NORMAL);
			break; }
		case IDM_CHAINVIEW_OPEN:
			_chainopen();
			break;
		case IDM_CHAINVIEW_SAVE:
			_chainsave();
			break;
		case IDM_CHAINVIEW_EXIT:
			DestroyWindow(hWnd);
			break;
		case IDM_CHAINVIEW_SETTINGS:
			HWND hDlg = CreateDialogParam( inst->getHInst(), MAKEINTRESOURCE(IDD_CHAINVIEW_SETTINGS), 
				hWnd, ChainView_SettingsDlgProc, (LPARAM)this);
			ShowWindow(hDlg,SW_NORMAL);
			break;
		}
		break;
	case WM_KEYDOWN:
		switch(wParam) {
		case 0x78:		// F9;
			render();
			break;
		case 0x77:		// F8;
			ctx->monitor.abortchilds();
			break;
		}
		break;
	case WM_TIMER:
		switch(wParam) {
		case STATUS_UPD_TIMER: {
			WCHAR str[64];
			double per = ctx->monitor.getpers();
			swprintf(str,L"%0.1lf %%",per*100);
			SendMessage(ctx->hStatusBar,WM_SETTEXT,0,(LPARAM)str);
			break; }
		}
		break;
	case WM_LBUTTONDOWN:
		SetFocus(hWnd);
		break;
	default:
		return DefWindowProc(hWnd,message,wParam,lParam);
	}
	return 0;
}

LRESULT HChainView::_wndproc_notify(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	switch( ((LPNMHDR)lParam)->code ) {
    case LVN_GETDISPINFO:
		_dispinfoListView(lParam);
		break;
	case NM_DBLCLK: {
		LPNMITEMACTIVATE item = (LPNMITEMACTIVATE) lParam;
		if( item->iItem!=-1 ) {
			HOpticalElement* elem = chain->getElement(item->iItem);
			elem->configWnd(inst,hWnd);
		}
		break; }
	case NM_RCLICK: {
		LPNMITEMACTIVATE item1 = (LPNMITEMACTIVATE) lParam;
		ctx->lastChainItem = item1->iItem;
		HMENU menu = CreatePopupMenu();//CreateMenu();
		MENUITEMINFO item;
		item.cbSize = sizeof(MENUITEMINFO);
		item.fMask = MIIM_STRING | MIIM_FTYPE | MIIM_ID;
		item.fType = MFT_STRING;
		item.wID = IDC_CHAINVIEW_EDITITEM;
		item.dwTypeData = L"Edit";
		InsertMenuItem(menu,0,true,&item);
		item.wID = IDC_CHAINVIEW_INSERTITEM;
		item.dwTypeData = L"Insert";
		InsertMenuItem(menu,1,true,&item);
		item.wID = IDC_CHAINVIEW_DELITEM;
		item.dwTypeData = L"Delete";
		InsertMenuItem(menu,2,true,&item);
		item.wID = IDC_CHAINVIEW_MOVEITEM;
		item.dwTypeData = L"Move";
		InsertMenuItem(menu,3,true,&item);
		item.wID = IDC_CHAINVIEW_CONFIG;
		item.dwTypeData = L"Configure";
		InsertMenuItem(menu,4,true,&item);
		POINT cursor;
        GetCursorPos(&cursor);
		TrackPopupMenuEx(menu,TPM_LEFTALIGN,cursor.x,cursor.y,hWnd,0);
		break; }
	}
	return 0;
}

bool HChainView::_initListView() {
	int res;
	if( ctx->hWndListView != NULL )
		DestroyWindow(ctx->hWndListView);
	HWND hWndListView = ctx->hWndListView =
		CreateWindow( WC_LISTVIEW, L"",
			WS_CHILD|LVS_REPORT|LVS_EDITLABELS,
			0, 0, 300, 300, ctx->hWnd, 0, ctx->hInst, NULL );
	ListView_SetExtendedListViewStyleEx(hWndListView, LVS_EX_FULLROWSELECT, LVS_EX_FULLROWSELECT);
	LVCOLUMN lvc;
	lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	lvc.iSubItem = 0;
    lvc.pszText = L"Id";
    lvc.cx = 30;
    lvc.fmt = LVCFMT_LEFT;
	res = ListView_InsertColumn(hWndListView,0,&lvc);
	lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	lvc.iSubItem = 1;
    lvc.pszText = L"Type";
    lvc.cx = 100;
    lvc.fmt = LVCFMT_LEFT;
	res = ListView_InsertColumn(hWndListView,1,&lvc);
	lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	lvc.iSubItem = 2;
    lvc.pszText = L"Name";
    lvc.cx = 100;
    lvc.fmt = LVCFMT_LEFT;
	res = ListView_InsertColumn(hWndListView,2,&lvc);

	int cnt = chain->countElements();
	for( int i=0; i<cnt; i++ ) {
		LVITEM lvi;
		lvi.pszText	  = LPSTR_TEXTCALLBACK;
		lvi.mask      = LVIF_TEXT | LVIF_STATE;
		lvi.stateMask = 0;
		lvi.iSubItem  = 0;
		lvi.state     = 0;
		lvi.iItem  = i;
		res = ListView_InsertItem(hWndListView,&lvi);
	}
	
	ShowWindow(hWndListView,SW_NORMAL);
	return true;
}

bool HChainView::_dispinfoListView(LPARAM lParam) {
	NMLVDISPINFO* pdi = (NMLVDISPINFO*) lParam;
	switch( pdi->item.iSubItem ) {
	case 0:
		wsprintf(ctx->szBuffer,L"%d",pdi->item.iItem+1);
		pdi->item.pszText = ctx->szBuffer;
		break;
	case 1: {
		HOpticalElement *elem = chain->getElement(pdi->item.iItem);
		//MultiByteToWideChar(CP_UTF8,0,elem->getTypeName(),-1,ctx->szBuffer,512);
		pdi->item.pszText = elem->getTypeName().bytes();
		break; }
	case 2: {
		HOpticalElement *elem = chain->getElement(pdi->item.iItem);
		//MultiByteToWideChar(CP_UTF8,0,elem->getTagName(),-1,ctx->szBuffer,512);
		pdi->item.pszText = elem->getTagName().bytes();
		break; }
	}
	return true;
}

LRESULT HChainView::_insertdlgproc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
	wchar_t str[256];
	//WCHAR buf[256];
	switch(message) {
	case WM_INITDIALOG: {
		swprintf(str,L"%0.2lf",0.0);
		int ntypes = inst->getElementClasses(0);
		ctx->optClasses = new const HOpticalElementClass*[ntypes];
		inst->getElementClasses(ctx->optClasses);
		for( int i=0; i<ntypes; i++ ) {
			//MultiByteToWideChar(CP_UTF8,0,ctx->optClasses[i]->nmType,-1,buf,256);
			ComboBox_InsertString( GetDlgItem(hDlg,IDD_CHAINVIEW_CLASS), i, ctx->optClasses[i]->nmType );
		}
		return (INT_PTR)TRUE;
	}
	case WM_COMMAND:
		switch( LOWORD(wParam) ) {
		case IDD_CHAINVIEW_INSERT: {
			GetDlgItemText(hDlg,IDD_CHAINVIEW_NAME,str,sizeof(str));
			int clsid = ComboBox_GetCurSel(GetDlgItem(hDlg,IDD_CHAINVIEW_CLASS));
			if( clsid!=-1 ) {
				HOpticalElement* elem = ctx->optClasses[clsid]->createElement(str);
				chain->addElement(elem,ctx->lastChainItem);
				LVITEM lvi;
				lvi.pszText	  = LPSTR_TEXTCALLBACK;
				lvi.mask      = LVIF_TEXT | LVIF_STATE;
				lvi.stateMask = 0;
				lvi.iSubItem  = 0;
				lvi.state     = 0;
				if( ctx->lastChainItem==-1 )
					lvi.iItem     = ListView_GetItemCount(ctx->hWndListView);
				else
					lvi.iItem     = ctx->lastChainItem;
				ListView_InsertItem(ctx->hWndListView,&lvi);
			}
			DestroyWindow(hDlg);
			//EndDialog(hDlg, LOWORD(wParam));
			return 1; }
		case IDD_CHAINVIEW_CLASS:
			if( HIWORD(wParam) == CBN_SELCHANGE ) {
				return 2;
			}
			return 1;
		}
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
		{
			//EndDialog(hDlg, LOWORD(wParam));
			HWND hParentWnd = GetParent(hDlg);
			DestroyWindow(hDlg);
			SetFocus(hParentWnd);
			return (INT_PTR)TRUE;
		}
		break;
	case WM_DESTROY: {
		HWND hParentWnd = GetParent(hDlg);
		SetFocus(hParentWnd);
		ChainView_Wnds.erase(hDlg);
		delete[] ctx->optClasses;
		return 0; }
	default:
		return DefWindowProc(hDlg,message,wParam,lParam);
	}
	return 0;
}

LRESULT HChainView::_elconfdlgproc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
	wchar_t str[256];
	//WCHAR buf[256];
	HOpticalElement *elem = ctx->configElements[hDlg];
	switch(message) {
	case WM_INITDIALOG: {
		if( ctx->lastChainItem == -1 ) {
			DestroyWindow(hDlg);
			return 0;
		}
		elem = chain->getElement(ctx->lastChainItem);
		ctx->configElements[hDlg] = elem;

		wchar_t* confdata;
		size_t sz_confdata;
		csrutils::DOMSObject config;
		elem->getconfig(config);
		csrutils::DOMSWrite(confdata,sz_confdata,config,csrutils::DOMSWriteFlags_NullTerm);
		SetDlgItemText(hDlg,IDC_ELEMCONFIG_CLASS,elem->getTypeName().bytes());
		SetDlgItemText(hDlg,IDC_ELEMCONFIG_NAME,elem->getTagName().bytes());
		SetDlgItemText(hDlg,IDC_ELEMCONFIG_CONFIG,confdata);
		return (INT_PTR)TRUE;
	}
	case WM_COMMAND:
		if( LOWORD(wParam)==IDOK || LOWORD(wParam)==IDAPPLY ) {
			GetDlgItemText(hDlg,IDC_ELEMCONFIG_NAME,str,sizeof(str));
			elem->setTagName(str);
			size_t sz_confdata = GetWindowTextLength( GetDlgItem(hDlg,IDC_ELEMCONFIG_CONFIG) );
			wchar_t* confdata = new wchar_t[sz_confdata+1];
			sz_confdata = GetDlgItemText(hDlg,IDC_ELEMCONFIG_CONFIG,confdata,(int)(sz_confdata+1));
			csrutils::DOMSObject config;
			csrutils::DOMSRead(confdata,sz_confdata,config,csrutils::DOMSWriteFlags_NullTerm);
			elem->setconfig(config);
			UpdateWindow(ctx->hWnd);
			ListView_Update(ctx->hWndListView,chain->positionElement(elem));
			render();
		}
		if(LOWORD(wParam)==IDOK || LOWORD(wParam)==IDCANCEL) {
			//EndDialog(hDlg, LOWORD(wParam));
			HWND hParentWnd = GetParent(hDlg);
			DestroyWindow(hDlg);
			SetFocus(hParentWnd);
			return (INT_PTR)TRUE;
		}
		break;
	case WM_DESTROY: {
		HWND hParentWnd = GetParent(hDlg);
		SetFocus(hParentWnd);
		ChainView_Wnds.erase(hDlg);
		return 0; }
	default:
		return DefWindowProc(hDlg,message,wParam,lParam);
	}
	return 0;
}

LRESULT HChainView::_elmovedlgproc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
	HOpticalElement *elem = ctx->configElements[hDlg];
	switch(message) {
	case WM_INITDIALOG: {
		elem = chain->getElement(ctx->lastChainItem);
		ctx->configElements[hDlg] = elem;
		int cur_pos = chain->positionElement(elem);
		SetDlgItemInt(hDlg,IDC_ELEMMOVE_POS,cur_pos+1,TRUE);
		return (INT_PTR) TRUE;
	}
	case WM_COMMAND:
		if( LOWORD(wParam) == IDOK ) {
			int cur_pos = chain->positionElement(elem);
			int new_pos = GetDlgItemInt(hDlg,IDC_ELEMMOVE_POS,0,TRUE);
			new_pos = new_pos - 1;
			if( new_pos != cur_pos ) {
				if( new_pos > cur_pos )
					new_pos --;
				chain->excludeElement(cur_pos);
				chain->addElement(elem,new_pos);
			}
			for( int i=0; i<chain->countElements(); i++ )
				ListView_Update(ctx->hWndListView,i);
		}
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
			HWND hParentWnd = GetParent(hDlg);
			DestroyWindow(hDlg);
			SetFocus(hParentWnd);
			return (INT_PTR) TRUE;
		}
		break;
	case WM_DESTROY: {
		HWND hParentWnd = GetParent(hDlg);
		SetFocus(hParentWnd);
		ChainView_Wnds.erase(hDlg);
		return 0; }
	default:
		return DefWindowProc(hDlg,message,wParam,lParam);
	}
	return 0;
}

LRESULT HChainView::_settingsdlgproc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
	//char str[256];
	//WCHAR buf[256];
	switch(message) {
	case WM_INITDIALOG: {
		wchar_t* confdata;
		size_t sz_confdata;
		csrutils::DOMSObject config;
		chain->getconfig(config);
		csrutils::DOMSWrite(confdata,sz_confdata,config,csrutils::DOMSWriteFlags_NullTerm);
		//SetDlgItemTextA(hDlg,IDC_ELEMCONFIG_CLASS,L"");
		//SetDlgItemTextA(hDlg,IDC_ELEMCONFIG_NAME,elem->getTagName());
		SetDlgItemText(hDlg,IDC_CHAINVIEW_ALLCONFIG,confdata);
		return (INT_PTR)TRUE;
	}
	case WM_COMMAND:
		if( LOWORD(wParam)==IDOK || LOWORD(wParam)==IDAPPLY ) {
			//GetDlgItemTextA(hDlg,IDC_ELEMCONFIG_NAME,str,sizeof(str));
			//elem->setTagName(str);
			size_t sz_confdata = GetWindowTextLength( GetDlgItem(hDlg,IDC_CHAINVIEW_ALLCONFIG) );
			wchar_t* confdata = new wchar_t[sz_confdata+1];
			sz_confdata = GetDlgItemText(hDlg,IDC_CHAINVIEW_ALLCONFIG,confdata,(int)(sz_confdata+1));
			csrutils::DOMSObject config;
			csrutils::DOMSRead(confdata,sz_confdata,config,csrutils::DOMSWriteFlags_NullTerm);
			chain->setconfig(config);
		}
		if(LOWORD(wParam)==IDOK || LOWORD(wParam)==IDCANCEL) {
			//EndDialog(hDlg, LOWORD(wParam));
			HWND hParentWnd = GetParent(hDlg);
			DestroyWindow(hDlg);
			SetFocus(hParentWnd);
			return (INT_PTR)TRUE;
		}
		break;
	case WM_DESTROY: {
		HWND hParentWnd = GetParent(hDlg);
		SetFocus(hParentWnd);
		ChainView_Wnds.erase(hDlg);
		return 0; }
	default:
		return DefWindowProc(hDlg,message,wParam,lParam);
	}
	return 0;
}

void HChainView::_chainsave() {
	csrutils::StringB path;
	bool res = QueryFileDialogue(ctx->hWnd,FILEDLG_SAVE,path,ChainView_FileTypes,L"chain");
	if(!res)
		return;
	File file(path);
	chain->save(file);
}

/* Open Chain from file
 * Opens file-choose dialogue,
 * or [commed] if `def`=true opens "chain.chain" file */
void HChainView::_chainopen(bool def) {
	csrutils::StringB path;
	//if( !def ) {
		bool res = QueryFileDialogue(ctx->hWnd,FILEDLG_OPEN,path,ChainView_FileTypes,L"chain");
		if(!res)
			return;
	//} else
	//	path = L"chain.chain";
	File file(path);
	chain->open(file);
	_initListView();
}
