#define _USE_MATH_DEFINES

#include <math.h>
#include <csrutils.h>
#include <csrmath.h>

#include "fft.h"

struct FFT_PRETABLE {
	int N;
	double *data;
};

struct FFT_DATA {
	double *src;
	double *buf;
	double *dst;
	int size;
	int idrow;	// idcol
	int step;
	int shift;
	int type;
	FFT_PRETABLE *table;
};

void fft_routine_row(FFT_DATA *dt);
void fft_precalv(FFT_PRETABLE *table, int N);

FFT_PRETABLE cahce_pretable;


void fft_2d(int size, double *src, double *dst) {
	fft_precalv(&cahce_pretable,size);
	FFT_DATA dt;
	dt.src = src;
	dt.buf = dst;
	dt.dst = new double[2*size*size];
	dt.size  = size;
	dt.step  = 1;
	dt.table = &cahce_pretable;
	dt.type = 0;
	for( int i=0; i<size; i++ ) {
		dt.idrow = i;
		dt.shift = 0;
		fft_routine_row(&dt);
	}
	dt.src = dt.buf;
	dt.buf = src; // !!!
	dt.dst = dst;
	dt.type = 1;
	for( int i=0; i<size; i++ ) {
		dt.idrow = i;
		dt.shift = 0;
		//fft_routine_col(&dt);
	}
	//delete[] dt.src;
}

void ifft_2d(int szx, int szy, double *src, double *dst) {
}

void fft_routine_row(FFT_DATA *dt) {
	int off, Off;
	if( dt->size <= dt->step ) {
		int k =0;
		if( dt->type ) { // cols
			off = 2*( dt->size*dt->shift + dt->idrow  );
		} else { // rows
			off = 2*( dt->size*dt->idrow + dt->shift );
		}
		dt->dst[off+0] = dt->buf[off+0];
		dt->dst[off+1] = dt->buf[off+1];
		return;
	}
	FFT_DATA ldt = *dt;
	ldt.dst = dt->buf;
	ldt.buf = dt->dst;
	ldt.step = 2*dt->step;
	ldt.shift = dt->shift + 0;
	fft_routine_row(&ldt);
	ldt.shift = dt->shift + dt->step;
	fft_routine_row(&ldt);
	int N_2 = dt->size / dt->step / 2;
	for( int k=0; k<N_2; k++ ) {
		double S_r = dt->table->data[k+0];
		double S_i = dt->table->data[k+1];
		if( dt->type ) { // cols
			off = 2*( dt->size*(2*k*dt->step + dt->shift) + dt->idrow  );
			Off = 2*( dt->size*(k*dt->step + dt->shift) + dt->idrow );
		} else { // rows
			off = 2*( dt->size*dt->idrow + 2*k*dt->step + dt->shift );
			Off = 2*( dt->size*dt->idrow + k*dt->step + dt->shift );
		}
		dt->dst[Off+0] = dt->buf[off+0] + S_r*dt->buf[off+2] - S_i*dt->buf[off+2];
		dt->dst[Off+1] = dt->buf[off+1] + S_r*dt->buf[off+3] + S_i*dt->buf[off+3];
	}
	for( int k=0; k<N_2; k++ ) {
		double S_r = dt->table->data[k+0];
		double S_i = dt->table->data[k+1];
		if( dt->type ) { // cols
			off = 2*( dt->size*(2*k*dt->step + dt->shift) + dt->idrow  );
			Off = 2*( dt->size*((k+N_2)*dt->step + dt->shift) + dt->idrow );
		} else { // rows
			off = 2*( dt->size*dt->idrow + 2*k*dt->step + dt->shift );
			Off = 2*( dt->size*dt->idrow + (k+N_2)*dt->step + dt->shift );
		}
		dt->dst[Off+0] = dt->buf[off+0] + S_r*dt->buf[off+2] - S_i*dt->buf[off+2];
		dt->dst[Off+1] = dt->buf[off+1] + S_r*dt->buf[off+3] + S_i*dt->buf[off+3];
	}
}

void fft_precalv(FFT_PRETABLE *table, int N) {
	int N2 = N*N;
	table->data = new double[2*N2];
	for( int i=0; i<N2; i++ ) {
		double f = 2.0*M_PI*i/N;
		table->data[2*i+0] = cos(f);
		table->data[2*i+1] = sin(f);
	}
}