#define _HUYGENS_INT_

#include "stdafx.h"
#include "hoptelems.h"
#include "lightvideo.h"
#include <vector>

using namespace csrutils;

struct HAnimator_Ctx {
};

struct HChainVideo_Ctx {
	HWaveView* view;
	HOpticalElement* space;
	HLightMap* map;
	double space_min;
	double space_max;
	uint32_t nsteps;
	uint32_t step;
	ThreadMonitor monitor;
};

HAnimator::HAnimator(HInstance* _inst, HOpticalChain* _chain)
		: inst(_inst), chain(_chain) {
	ctx = new HAnimator_Ctx;
}

HAnimator::~HAnimator() {
	delete ctx;
}

void HAnimator::run(HVideo* video) {
	HLightMap* map = video->getmap();
	HDC hdc = GetDC(NULL);
	//ctx->image = NULL;
	HDC ihdc = CreateCompatibleDC(hdc);
	HDC ihdc2 = CreateCompatibleDC(hdc);
	int vid_width  = 512;
	int vid_height = 512;
	HBITMAP ihbm = NULL;
	HBITMAP ihbm2 = NULL;
	Image* img = new Image(map->width,map->height,csrutils::ImageBitFormat_RGB24,ihdc,ihbm);
	Image* img2 = new Image(vid_width,vid_height,csrutils::ImageBitFormat_RGB24,ihdc,ihbm2);
	SelectObject(ihdc,ihbm);
	SelectObject(ihdc2,ihbm2);
	uint32_t frame;
	uint32_t nframes = video->getframescount();
	ThreadMonitor monitor;
	double dist_min = 38.0;
	double dist_max = 7.6;
	ImageEncoderParams img_param;
	img_param.mask = ImageEncoderParams_Quality;
	img_param.Quality = 90;
	for( frame=0; frame<nframes; frame++ ) {
		double dist = dist_min + (dist_max-dist_min)*frame/nframes;
		video->getframe(map);
		Huygens_RenderLightMap(map,img,0.075,HRT_AMPL);
		Huygens_CopyWithScale(ihdc2,ihdc,vid_width,vid_height,img->getWidth(),img->getHeight());
		wchar_t path[256];
		swprintf(path,L"d=%0.1lfmm",dist);
		TextOut(ihdc2,10,10,path,(int)wcslen(path));
		swprintf(path,L"D:\\Temp\\huygens\\frames\\%03d.jpg",frame);
		img2->save(path,ImageFileFormat_JPEG,&img_param);
		video->nextframe();
	}
	delete img;
	delete img2;
	DeleteDC(ihdc);
	DeleteDC(ihdc2);
	DeleteObject(ihbm2);
}

void HAnimator::render_lightvideo(HVideo* video) {
	HLightMap* map = video->getmap();
	HLightVideoWriter* lvid
		= new HLightVideoWriter( File(L"D:\\Temp\\huygens\\1.lvid"), map->width, map->height );
	uint32_t frame;
	uint32_t nframes = video->getframescount();
	for( frame=0; frame<nframes; frame++ ) {
		video->getframe(map);
		lvid->writeframe(map);
		video->nextframe();
	}
	delete lvid;
}

HChainVideo::HChainVideo(HOpticalChain* _chain) : chain(_chain) {
	ctx = new HChainVideo_Ctx;
	ctx->view = (HWaveView*) chain->getElementByName(L"anim_view");
	if( ctx->view==NULL )
		throw;
	//ctx->space = (HOpticalElement*) chain->getElementByName(L"anim_space");
	ctx->space = (HOpticalElement*) chain->getElementByName(L"aperture");
	if( ctx->space==NULL )
		throw;
	ctx->monitor.setjob(0.0);
	chain->render(ctx->monitor);
	ctx->monitor.waitforchilds();

	ctx->map = ctx->view->getmap();
	ctx->space_min = 0.25;
	ctx->space_max = 0.05;
	ctx->nsteps = 100;
	ctx->step = -1;
	nextframe();
}

HChainVideo::~HChainVideo() {
	delete ctx;
}

bool HChainVideo::getframe(HLightMap* map) {
	return true;
}

uint32_t HChainVideo::getframescount() {
	return ctx->nsteps;
}

uint32_t HChainVideo::getframeid() {
	return ctx->step;
}

bool HChainVideo::nextframe() {
	ctx->step ++;
	double dist = ctx->space_min + (ctx->space_max-ctx->space_min)*ctx->step/ctx->nsteps;
	DOMSObject space_c;
	ctx->space->getconfig(space_c);
	//space_c["depth"] = dist;
	space_c[L"radius"] = dist;
	ctx->space->setconfig(space_c);
	ctx->monitor.setjob(0.0);
	chain->render(ctx->monitor);
	ctx->monitor.waitforchilds();
	return true;
}

bool HChainVideo::seekframe(uint32_t frameid) {
	ctx->step = frameid;
	return true;
}

HLightMap* HChainVideo::getmap() {
	return ctx->map;
}

/*
	HWaveView* view = (HWaveView*) chain->getElementByName(L"anym_view");
	if( view==NULL )
		return;
	HOpticalElement* space = (HOpticalElement*) chain->getElementByName(L"anym_space");
	if( space==NULL )
		return;
	HLightMap* map = view->getmap();
	HDC hdc = GetDC(NULL);
	//ctx->image = NULL;
	HDC ihdc = CreateCompatibleDC(hdc);
	HBITMAP ihbm = NULL;
	Image* img = new Image(map->width,map->height,csrutils::ImageBitFormat_RGB24,ihdc,ihbm);
	SelectObject(ihdc,ihbm);
	int frame;
	int nframes = 800;
	ThreadMonitor monitor;
	double dist_min = 1.0;
	double dist_max = 5.0;
	for( frame=0; frame<nframes; frame++ ) {
		double dist = dist_min + (dist_max-dist_min)*frame/nframes;
		DOMSObject space_c;
		space->getconfig(space_c);
		space_c["depth"] = dist;
		space->setconfig(space_c);
		monitor.setjob(0.0);
		chain->render(monitor);
		monitor.waitforchilds();
		Huygens_RenderLightMap(map,img,0.25,HRT_AMPL);
		wchar_t path[256];
		swprintf(path,L"d=%0.2lf",dist);
		TextOut(ihdc,10,10,path,(int)wcslen(path));
		swprintf(path,L"D:\\Temp\\huygens\\f_%03d.jpg",frame);
		img->save(path,ImageFileFormat_JPEG);
	}
	delete img;
	CloseHandle(hdc);
}*/