#define _HUYGENS_INT_
#define _CRT_SECURE_NO_WARNINGS
#include "stdafx.h"
#include "hoptelems.h"
#include <map>
#include <csrmath.h>
#include <csrmath_complex.h>
#include "resource.h"

using namespace csrutils;
using csrmath::ComplexD;
using csrmath::expi;

#define PROFILE_RECT		0
#define PROFILE_CONT		1
#define PROFILE_INTRECT		2
#define PROFILE_INVINTRECT	-2

/**************************************
 **        Class HBesselPlate        **
 **************************************/

HOpticalElementClass *clsHBesselPlate = 0;
const wchar_t* szClsHBesselPlate = L"BesselPlate";

HOpticalElement* createHBesselPlate(const String& name) {
	return new HBesselPlate(name);
}

const HOpticalElementClass* getHBesselPlateClass() {
	if( clsHBesselPlate==0 ) {
		clsHBesselPlate = new HOpticalElementClass;
		clsHBesselPlate->nmType = szClsHBesselPlate;
		clsHBesselPlate->createElement = createHBesselPlate;
	}
	return clsHBesselPlate;
}

HBesselPlate::HBesselPlate(const String& nmTag)
		: HOpticalElement(szClsHBesselPlate,nmTag), wavelength(0.0005), focuslength(10.0),
		index(1), profile(PROFILE_RECT), angle(0.0) {
}

HBesselPlate::HBesselPlate(const String& nmTag, double focuslength, double wavelength, int index)
		: HOpticalElement(szClsHBesselPlate,nmTag), focuslength(focuslength),
		  wavelength(wavelength), index(index), profile(PROFILE_RECT) {
}

void HBesselPlate::run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon) {
	for( uint32_t x=0; x<output.width; x++ )
	for( uint32_t y=0; y<output.width; y++ ) {
		double X = input.getX(x);
		double Y = input.getY(y);
		double theta = atan2(Y,X) + angle;
		double R2 = X*X+Y*Y;
		double R = sqrt(R2);
		double s;
		if( focuslength != 0.0 )
				// Spiral Fresnel zone plate (focuslength!=0.0)
			s = R2 / (focuslength*wavelength) + index*theta/(2.0*M_PI);
		else
				// Spiral axicon (focuslength==0.0)
			s = index*theta/(2.0*M_PI) + R/(27.0*wavelength);
		switch( profile ) {
		case PROFILE_RECT: {
			int m = (int) floor( 2.0*s );
			if( m%2==0 ) {
				output.data[ 2*(x+y*output.width) + 0 ] = input.data[ 2*(x+y*output.width) + 0 ];
				output.data[ 2*(x+y*output.width) + 1 ] = input.data[ 2*(x+y*output.width) + 1 ];
			} else {
				output.data[ 2*(x+y*output.width) + 0 ] = - input.data[ 2*(x+y*output.width) + 0 ];
				output.data[ 2*(x+y*output.width) + 1 ] = - input.data[ 2*(x+y*output.width) + 1 ];
			}
			break; }
		case PROFILE_CONT: {
			*(ComplexD*)&output.data[ 2*(x+y*output.width)]
				= expi(s*(2.0*M_PI)) * *(ComplexD*)&input.data[2*(x+y*output.width)];
			break; }
		case PROFILE_INTRECT:
		case PROFILE_INVINTRECT: {
			int m = (int) floor( 2.0*s );
			if(profile==PROFILE_INVINTRECT)
				m ++;
			if( m%2==0 ) {
				output.data[ 2*(x+y*output.width) + 0 ] = input.data[ 2*(x+y*output.width) + 0 ];
				output.data[ 2*(x+y*output.width) + 1 ] = input.data[ 2*(x+y*output.width) + 1 ];
			} else {
				output.data[ 2*(x+y*output.width) + 0 ] = 0.0;
				output.data[ 2*(x+y*output.width) + 1 ] = 0.0;
			}
			break; }
		default: {
			double n = profile;
			//int m = (int) floor( 3.0*s );
			double p = (2.0/n*M_PI) * floor( n*s );
			*(ComplexD*)&output.data[ 2*(x+y*output.width)]
				= expi(p) * *(ComplexD*)&input.data[2*(x+y*output.width)];
			break; }
		}
	}
}

bool HBesselPlate::getconfig(csrutils::DOMSObject& st) {
	st[L"focuslength"] = focuslength;
	st[L"wavelength"]  = wavelength;
	st[L"index"]       = index;
	st[L"profile"]     = profile;
	if( angle != 0.0 )
		st[L"angle"]   = angle;
	return true;
}

bool HBesselPlate::setconfig(csrutils::DOMSObject& st) {
	focuslength = st[L"focuslength"];
	wavelength  = st[L"wavelength"];
	index       = st[L"index"];
	profile     = st[L"profile"].exsists() ? st[L"profile"] : PROFILE_RECT;
	angle       = st[L"angle"].exsists() ? st[L"angle"] : 0.0;
	return true;
}
