#define _HUYGENS_INT_
#define _CRT_SECURE_NO_WARNINGS
#include "stdafx.h"
#include "hoptelems.h"
#include <map>
#include <csrmath.h>
#include <csrmath_complex.h>
#include "resource.h"

using namespace csrutils;
using csrmath::ComplexD;
using csrmath::expi;

extern "C" double besseljn(const __int64 n, const double x);

HOpticalElementClass *clsHBesselWave = 0;
const wchar_t* szClsHBesselWave = L"BesselWave";

HOpticalElement* createHBesselWave(const String& name) {
	return new HBesselWave(name);
}

const HOpticalElementClass* getHBesselWaveClass() {
	if( clsHBesselWave==0 ) {
		clsHBesselWave = new HOpticalElementClass;
		clsHBesselWave->nmType = szClsHBesselWave;
		clsHBesselWave->createElement = createHBesselWave;
	}
	return clsHBesselWave;
}

HBesselWave::HBesselWave(const String& nmTag)
		: HOpticalElement(szClsHBesselWave,nmTag)  {
	intensity = 1.0;
	phase     = 0.0;
	radius    = 0.2;
	index     = 1;
}

void HBesselWave::run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon) {
	double r2 = radius*radius;
	int Index = index > 0 ? index : -index;
	for( uint32_t x=0; x<output.width; x++ )
	for( uint32_t y=0; y<output.width; y++ ) {
		double X = input.getX(x);
		double Y = input.getY(y);
		double R = sqrt( X*X + Y*Y );
		double P = atan2( Y, X );
		*(ComplexD*)&output.data[ 2*(x+y*output.width)]
				= intensity * besseljn( Index, R/radius ) * expi( P*index + phase );
	}
}

bool HBesselWave::getconfig(csrutils::DOMSObject& st) {
	st[L"intensity"] = intensity;
	st[L"phase"]     = phase;
	st[L"radius"]    = radius;
	st[L"index"]     = index;
	return true;
}

bool HBesselWave::setconfig(csrutils::DOMSObject& st) {
	intensity = st[L"intensity"];
	phase     = st[L"phase"];
	radius    = st[L"radius"];
	index     = st[L"index"];
	return true;
}