#define _HUYGENS_INT_

#include "stdafx.h"
#include <vector>

using namespace csrutils;

struct HOpticalChain_Elem {
	HOpticalElement* elem;
	//double Z;
	HLightMap *outmap;
	bool outmap_valid;
};

struct HOpticalChain_Ctx {
	std::vector<HOpticalChain_Elem> elements;
	int map_width, map_height;
	double map_minX, map_maxX, map_minY, map_maxY;
};

HOpticalChain::HOpticalChain(HInstance* _inst) : inst(_inst) {
	ctx = new HOpticalChain_Ctx;
	ctx->map_width  = 512;
	ctx->map_height = 512;
	ctx->map_minX = -0.5;
	ctx->map_maxX =  0.5;
	ctx->map_minY = -0.5;
	ctx->map_maxY =  0.5;
}

HOpticalChain::~HOpticalChain() {
	clear();
	delete ctx;
}

void HOpticalChain::addElement(HOpticalElement* element, int pos) {
	HOpticalChain_Elem el;
	el.elem   = element;
	//el.Z      = Z;
	el.outmap = NULL;
	el.outmap_valid = false;
	if( pos<0 )
		ctx->elements.push_back( el );
	else
		ctx->elements.insert( ctx->elements.begin()+pos, el );
	element->putInChain(this);
}

HOpticalElement* HOpticalChain::getElement(int pos) {
	return ctx->elements[pos].elem;
}

HOpticalElement* HOpticalChain::getElementByName(const csrutils::String& name) {
	for( auto iter = ctx->elements.begin(); iter < ctx->elements.end(); iter++ ) {
		if( (*iter).elem->getTagName() == name )
			return (*iter).elem;
	}
	return NULL;
}

int HOpticalChain::countElements() {
	return (int) ctx->elements.size();
}

int HOpticalChain::positionElement(HOpticalElement* element) {
	for( auto iter = ctx->elements.begin(); iter < ctx->elements.end(); iter++ )
		if( (*iter).elem == element )
			return (int) ( iter - ctx->elements.begin() );
	return -1;
}

void HOpticalChain::removeElement(int pos) {
	delete ctx->elements[pos].elem;
	ctx->elements.erase( ctx->elements.begin()+pos );
}

void HOpticalChain::clear() {
	for( auto iter = ctx->elements.begin(); iter < ctx->elements.end(); iter++ )
		delete (*iter).elem;
	ctx->elements.clear();
}

HOpticalElement* HOpticalChain::excludeElement(int pos) {
	HOpticalElement* el = ctx->elements[pos].elem;
	ctx->elements.erase( ctx->elements.begin()+pos );
	return el;
}

bool HOpticalChain_ThreadProc(csrutils::ThreadMonitor& monitor) {
	HOpticalChain* chain = (HOpticalChain*) monitor.getuserdata();
	chain->render_proc(monitor);
	return true;
}

void HOpticalChain::render(csrutils::ThreadMonitor& monitor) { //HLightMap& img, double Z) {
	csrutils::ThreadMonitorChild child = monitor.createchild(HOpticalChain_ThreadProc,1.0,this);
	child.run();
}

void HOpticalChain::render_proc(csrutils::ThreadMonitor& monitor) {
	//HLightMap* buffer0 = new HLightMap(img);
	//HLightMap* buffer1 = new HLightMap(img);
	HLightMap* buffer0 = new HLightMap( ctx->map_width, ctx->map_height, ctx->map_minX,ctx->map_minY,ctx->map_maxY, ctx->map_maxY );
	HLightMap* buffer1 = new HLightMap( ctx->map_width, ctx->map_height, ctx->map_minX, ctx->map_minY, ctx->map_maxY, ctx->map_maxY );
	bool last = false;
	//csrutils::ExecitionMonitor monitor;
	double partval = 1.0 / ctx->elements.size();
	for( auto iter = ctx->elements.begin(); iter < ctx->elements.end() && !last; iter++ ) {
		HOpticalChain_Elem el = *iter;
		monitor.pushjob(partval);
		el.elem->run(*buffer0,*buffer1,monitor);
		//Sleep(500);
		monitor.popjob();
		std::swap(buffer0,buffer1);
	}
	//img.copy(*buffer0);
	delete buffer0;
	delete buffer1;
}

bool HOpticalChain::getconfig(csrutils::DOMSObject& st) {
	st[L"map_width"]  = ctx->map_width;
	st[L"map_height"] = ctx->map_height;
	st[L"map_minX"] = ctx->map_minX;
	st[L"map_maxX"] = ctx->map_maxX;
	st[L"map_minY"] = ctx->map_minY;
	st[L"map_maxY"] = ctx->map_maxY;
	return true;
}

bool HOpticalChain::setconfig(csrutils::DOMSObject& st) {
	ctx->map_width  = st[L"map_width"];
	ctx->map_height = st[L"map_height"];
	ctx->map_minX = st[L"map_minX"];
	ctx->map_maxX = st[L"map_maxX"];
	ctx->map_minY = st[L"map_minY"];
	ctx->map_maxY = st[L"map_maxY"];
	return true;
}

bool HOpticalChain::open(File& file) {
	//csrutils::FileHandle fileh(file,csrutils::File_Read);
	csrutils::FileHandle* fileh = file.open(csrutils::File_Read);
	size_t store_sz = fileh->getsize();
	char *store_data = new char[store_sz];
	fileh->read(store_data,store_sz,store_sz);
	csrutils::DOMSObject store;
	store_sz = store.load(store_data,store_sz);
	clear();
	setconfig( store[L"config"].obj() );
	for( csrutils::DOMSIterator iter = store.iter(); iter; iter++ ) {
		csrutils::DOMSItem& sselem = *iter;
		if( sselem.isnamed() )
			continue;
		csrutils::DOMSObject& selem = sselem;
		csrutils::StringB cls_nm  = selem[L"class"].str();
		csrutils::StringB name_nm = selem[L"name"].str();
		const HOpticalElementClass* cls = inst->getElementClass( cls_nm.bytes() );
		if( cls==NULL ) {
			MessageBox(0,cls_nm.bytes(),L"Not found",0);
			return false;
		}
		HOpticalElement* elem = cls->createElement( name_nm.bytes() );
		elem->setconfig( selem[L"config"] );
		addElement(elem,-1);
	}
	heap_free(fileh);
	return true;
}

bool HOpticalChain::save(File& file) {
	csrutils::DOMSObject store;
	getconfig( store[L"config"].obj() );
	int count = countElements();
	for( int i=0; i<count; i++ ) {
		HOpticalElement* elem = getElement(i);
		csrutils::DOMSObject& selem = store[-1].obj();
		selem[L"class"] = elem->getTypeName();
		selem[L"name"]  = elem->getTagName();
		//selem["config"].make_object();
		elem->getconfig( selem[L"config"].obj() );
	}
	uint64_t store_sz = store.size();
	char *store_data = new char[store_sz];
	store_sz = store.store(store_data,store_sz);
	if( store_sz==0 ) {
		MessageBox(0,L"Error",L"",0);
		return false;
	}
	csrutils::FileHandle* fileh = file.open(csrutils::File_Write);
	fileh->write(store_data,store_sz,store_sz);
	heap_free(fileh);
	return true;
}