#include "huygens.h"

//using namespace csrutils;

void registerStdOpticalElements(HInstance* inst);

const HOpticalElementClass* getHWaveViewClass();
const HOpticalElementClass* getHPlanarWaveClass();
const HOpticalElementClass* getHBesselWaveClass();
const HOpticalElementClass* getHDifrDotClass();
const HOpticalElementClass* getHZonePlateClass();
const HOpticalElementClass* getHBesselPlateClass();
const HOpticalElementClass* getHThinLensClass();
const HOpticalElementClass* getHImageTranspClass();
const HOpticalElementClass* getHSpaceShiftClass();
const HOpticalElementClass* getHSpaceHuygensClass();
const HOpticalElementClass* getHSpaceFresnelClass();
const HOpticalElementClass* getHSpaceFourierClass();

class HWaveView : public HOpticalElement {
	HImageView* view;
	HLightMap* map;
	int iwidth;
	int iheight;

public:
	HWaveView(const csrutils::String& nmTag);

	void run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor& mon);
	//void putInChain(HOpticalChain* chain) { };
	HWND configWnd(HInstance* Inst, HWND hParentWnd);
	bool getconfig(csrutils::DOMSObject& st);
	bool setconfig(csrutils::DOMSObject& st);
	HLightMap* getmap();
};

class HPlanarWave : public HOpticalElement {
	double intensity;
	double phase;

public:
	HPlanarWave(const csrutils::String& nmTag);
	HPlanarWave(const csrutils::String& nmTag, double intensity, double phase);

	void run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon);
	//void putInChain(HOpticalChain* chain) { };
	HWND configWnd(HInstance *Inst, HWND hParentWnd);
	bool getconfig(csrutils::DOMSObject& st);
	bool setconfig(csrutils::DOMSObject& st);

#ifdef _HUYGENS_INT_
	LRESULT _wndproc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
#endif
};

class HBesselWave : public HOpticalElement {
	double intensity;
	double phase;
	int index;
	double radius;

public:
	HBesselWave(const csrutils::String& nmTag);
	//HBesselWave(const csrutils::String& nmTag, double intensity, double phase);

	void run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon);
	//void putInChain(HOpticalChain* chain) { };
	bool getconfig(csrutils::DOMSObject& st);
	bool setconfig(csrutils::DOMSObject& st);
};

class HDifrDot : public HOpticalElement {
	double radius;

public:
	HDifrDot(const csrutils::String& nmTag);
	HDifrDot(const csrutils::String& nmTag, double radius);

	void run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon);
	//void putInChain(HOpticalChain* chain) { };
	bool getconfig(csrutils::DOMSObject& st);
	bool setconfig(csrutils::DOMSObject& st);
};

class HZonePlate : public HOpticalElement {
	double focuslength;
	double wavelength;

public:
	HZonePlate(const csrutils::String& nmTag);
	HZonePlate(const csrutils::String& nmTag, double focuslength, double wavelength);

	void run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon);
	//void putInChain(HOpticalChain* chain) { };
	bool getconfig(csrutils::DOMSObject& st);
	bool setconfig(csrutils::DOMSObject& st);
};

class HBesselPlate : public HOpticalElement {
	double focuslength;
	double wavelength;
	double angle;
	int index;
	int profile;	// 0 - rect, 1 - continuous

public:
	HBesselPlate(const csrutils::String& nmTag);
	HBesselPlate(const csrutils::String& nmTag, double focuslength, double wavelength, int index);

	void run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon);
	//void putInChain(HOpticalChain* chain) { };
	bool getconfig(csrutils::DOMSObject& st);
	bool setconfig(csrutils::DOMSObject& st);
};


class HThinLens : public HOpticalElement {
	double focuslength;
	double wavelength;

public:
	HThinLens(const csrutils::String& nmTag);
	HThinLens(const csrutils::String& nmTag, double focuslength);

	void run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon);
	//void putInChain(HOpticalChain* chain) { };
	bool getconfig(csrutils::DOMSObject& st);
	bool setconfig(csrutils::DOMSObject& st);
};


class HImageTransp : public HOpticalElement {
	csrutils::Image* image;
	double img_w, img_h;
	csrutils::StringB filepath;

public:
	HImageTransp(const csrutils::String& nmTag);
	HImageTransp(const csrutils::String& nmTag, csrutils::Image* image);

	void run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon);
	//void putInChain(HOpticalChain* chain) { };
	bool getconfig(csrutils::DOMSObject& st);
	bool setconfig(csrutils::DOMSObject& st);
};


/*
Using Fresnel-Kirchhoff formula to calculate diffraction
*/
class HSpaceHuygens : public HOpticalElement {
	double depth;
	double wavelength;
	int restep;

public:
	HSpaceHuygens(const csrutils::String& nmTag);
	HSpaceHuygens(const csrutils::String& nmTag, double depth, double wavelength);
	void run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon);
	bool getconfig(csrutils::DOMSObject& st);
	bool setconfig(csrutils::DOMSObject& st);
};

/*
Using Fresnel-Helmholtz approximation of Fresnel-Kirchhoff formula to calculate diffraction
*/
class HSpaceFresnel : public HOpticalElement {
	double depth;
	double wavelength;
	int restep;

public:
	HSpaceFresnel(const csrutils::String& nmTag);
	HSpaceFresnel(const csrutils::String& nmTag, double depth, double wavelength);
	void run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon);
	bool getconfig(csrutils::DOMSObject& st);
	bool setconfig(csrutils::DOMSObject& st);
};

class HSpaceShift : public HOpticalElement {
	double depth;
	double wavelength;

public:
	HSpaceShift(const csrutils::String& nmTag);
	HSpaceShift(const csrutils::String& nmTag, double depth, double wavelength);
	void run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon);
	bool getconfig(csrutils::DOMSObject& st);
	bool setconfig(csrutils::DOMSObject& st);
};

class HSpaceFourier : public HOpticalElement {
	double depth;
	double wavelength;

public:
	HSpaceFourier(const csrutils::String& nmTag);
	HSpaceFourier(const csrutils::String& nmTag, double depth, double wavelength);
	void run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon);
	bool getconfig(csrutils::DOMSObject& st);
	bool setconfig(csrutils::DOMSObject& st);
};
