#define _HUYGENS_INT_
#define _CRT_SECURE_NO_WARNINGS
#include "stdafx.h"
#include "hoptelems.h"
#include <map>
#include <csrmath.h>
#include <csrmath_complex.h>
#include "resource.h"

using namespace csrutils;
using csrmath::ComplexD;
using csrmath::expi;

std::map<HWND,HPlanarWave*> HPlanarWave_Wnds;


HOpticalElementClass* clsHPlanarWave = 0;
const wchar_t* szClsHPlanarWave = L"PlanarWave";

HOpticalElement* createHPlanarWave(const String& name) {
	return new HPlanarWave(name);
}

const HOpticalElementClass* getHPlanarWaveClass() {
	if( clsHPlanarWave==0 ) {
		clsHPlanarWave = new HOpticalElementClass;
		clsHPlanarWave->nmType = szClsHPlanarWave;
		clsHPlanarWave->createElement = createHPlanarWave;
	}
	return clsHPlanarWave;
}

HPlanarWave::HPlanarWave(const String& nmTag)
		: HOpticalElement(szClsHPlanarWave,nmTag), intensity(1.0), phase(0.0) {
}

HPlanarWave::HPlanarWave(const String& nmTag, double intensity, double phase)
		: HOpticalElement(szClsHPlanarWave,nmTag), intensity(intensity), phase(phase) {
}

void HPlanarWave::run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon) {
	ComplexD c = intensity * expi(phase);
	for( uint32_t x=0; x<output.width; x++ )
	for( uint32_t y=0; y<output.width; y++ ) {
		//ComplexD c = intensity * expi( 2.0*M_PI * input.getX(x) );
		output.data[ 2*(x+y*output.width) + 0 ] = c.Re();
		output.data[ 2*(x+y*output.width) + 1 ] = c.Im();
	}
}

LRESULT CALLBACK HPlanarWave_ConfigProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
	switch(message) {
	case WM_INITDIALOG: {
		HPlanarWave* elem = (HPlanarWave*) lParam;
		HPlanarWave_Wnds[hDlg] = elem;
		return elem->_wndproc(hDlg,message,wParam,lParam); }
	default: {
		HPlanarWave* elem = HPlanarWave_Wnds[hDlg];
		return elem->_wndproc(hDlg,message,wParam,lParam); }
	}
}

HWND HPlanarWave::configWnd(HInstance *inst, HWND hParentWnd) {
	HWND hDlg = CreateDialogParam( inst->getHInst(), MAKEINTRESOURCE(IDD_PLANARWAVE_CONFIG), 
		hParentWnd, HPlanarWave_ConfigProc, (LPARAM)this);
	ShowWindow(hDlg,SW_NORMAL);
	return hDlg;
}

bool HPlanarWave::getconfig(csrutils::DOMSObject& st) {
	st[L"intensity"] = intensity;
	st[L"phase"] = phase;
	return true;
}

bool HPlanarWave::setconfig(csrutils::DOMSObject& st) {
	intensity = st[L"intensity"];
	phase = st[L"phase"];
	return true;
}

LRESULT HPlanarWave::_wndproc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
	wchar_t str[256];
	switch(message) {
	case WM_INITDIALOG: {
		swprintf(str,L"%0.2lf",intensity);
		SetDlgItemText(hDlg,IDC_PLANARWAVE_INTENS,str);
		swprintf(str,L"%0.2lf",phase);
		SetDlgItemText(hDlg,IDC_PLANARWAVE_PHASE,str);
		return (INT_PTR)TRUE;
	}
	case WM_COMMAND:
		switch(wParam) {
		case IDC_PLANARWAVE_APPLY:
			//HuygensTask_Abort(&huygnestask);
			GetDlgItemText(hDlg,IDC_PLANARWAVE_INTENS,str,sizeof(str));
			swscanf(str,L"%lf",&intensity);
			GetDlgItemText(hDlg,IDC_PLANARWAVE_PHASE,str,sizeof(str));
			swscanf(str,L"%lf",&phase);
			//HuygensTask_Start(&huygnestask);
			return 1;
		}
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
		{
			//EndDialog(hDlg, LOWORD(wParam));
			HWND hParentWnd = GetParent(hDlg);
			DestroyWindow(hDlg);
			SetFocus(hParentWnd);
			return (INT_PTR)TRUE;
		}
		break;
	case WM_CLOSE: {
		HWND hParentWnd = GetParent(hDlg);
		DestroyWindow(hDlg);
		SetFocus(hParentWnd);
		HPlanarWave_Wnds.erase(hDlg);
		return 0; }
	default:
		return DefWindowProc(hDlg,message,wParam,lParam);
	}
	return 0;
}
