#define _HUYGENS_INT_
#define _USE_MATH_DEFINES
#include "stdafx.h"
#include "hoptelems.h"
#include <math.h>
#include <csrmath.h>
#include <csrmath_complex.h>
#include <fftw3.h>
#include <stdlib.h>
#include <time.h>

using namespace csrutils;
using csrmath::ComplexD;
using csrmath::expi;

double rand_d() {
	return (double)rand() / RAND_MAX;
}


/**************************************
 **        Class HSpaceFourier       **
 **************************************/

HOpticalElementClass* clsHSpaceFourier = 0;
const wchar_t* szClsHSpaceFourier = L"SpaceFourier";

HOpticalElement* createHSpaceFourier(const String& name) {
	return new HSpaceFourier(name);
}

const HOpticalElementClass* getHSpaceFourierClass() {
	if( clsHSpaceFourier==0 ) {
		clsHSpaceFourier = new HOpticalElementClass;
		clsHSpaceFourier->nmType = szClsHSpaceFourier;
		clsHSpaceFourier->createElement = createHSpaceFourier;
	}
	return clsHSpaceFourier;
}

HSpaceFourier::HSpaceFourier(const String& nmTag)
		: HOpticalElement(szClsHSpaceFourier,nmTag), depth(10.0), wavelength(0.005) {
}

HSpaceFourier::HSpaceFourier(const String& nmTag, double _depth, double _wavelength)
		: HOpticalElement(szClsHSpaceFourier,nmTag), depth(_depth), wavelength(_wavelength) {
}

void HSpaceFourier::run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon) {
	double K = 2.0*M_PI / wavelength;
	double Z = depth;
	int W = (int) input.width;
	//srand( clock() );
	HLightMap buffer1(input);
	HLightMap buffer2(input);
	fftw_complex* fft_in = (fftw_complex*) input.data;
	fftw_complex* fft_out = (fftw_complex*) output.data;
	fftw_complex* fft_f1 = fftw_alloc_complex(W*W);
	fftw_complex* fft_f2 = (fftw_complex*) buffer1.data;//fftw_alloc_complex(W*W);
	fftw_complex* fft_func = (fftw_complex*) buffer2.data;
	ComplexD* c_fft_f1 = (ComplexD*) fft_f1;
	ComplexD* c_fft_f2 = (ComplexD*) fft_f2;
	ComplexD* c_fft_func = (ComplexD*) fft_func;
	ComplexD* c_fft_out = (ComplexD*) fft_out;
	for( int x=0; x<W; x++ )
	for( int y=0; y<W; y++ ) {
		double X = input.getX(x);// + rand_d()/W/10;
		double Y = input.getY(y);// + rand_d()/W/10;
		c_fft_func[x+W*y] = expi( K/(2*Z)*( X*X + Y*Y ) );
		//double R = sqrt( Z*Z + X*X + Y*Y );
		//c_fft_out[x+W*y] = expi( K*R ) / R * (Z/R);
	}
	//buffer2.swapquarters();
	fftw_plan p;
	p = fftw_plan_dft_2d(W,W,fft_in,fft_f1,FFTW_FORWARD,FFTW_MEASURE);
	fftw_execute(p);
	fftw_destroy_plan(p);
	mon.setjob(0.333);
	p = fftw_plan_dft_2d(W,W,fft_func,fft_f2,FFTW_FORWARD,FFTW_MEASURE);
	fftw_execute(p);
	fftw_destroy_plan(p);
	//
	/*for( int x=0; x<W; x++ )
	for( int y=0; y<W; y++ ) {
		double X = input.getX(x);// + rand_d()/W/10;
		double Y = input.getY(y);// + rand_d()/W/10;
		c_fft_f2[x+W*y] = expi( M_PI * wavelength * Z * (X*X+Y*Y) );
		//double R = sqrt( Z*Z + X*X + Y*Y );
		//c_fft_out[x+W*y] = expi( K*R ) / R * (Z/R);
	}*/
	//buffer1.swapquarters();
	mon.setjob(0.666);
	for( int x=0; x<W; x++ )
	for( int y=0; y<W; y++ ) {
		c_fft_func[x+W*y] = c_fft_f1[x+W*y] * c_fft_f2[x+W*y];
	}
	p = fftw_plan_dft_2d(W,W,fft_func,fft_out,FFTW_BACKWARD,FFTW_MEASURE);
	fftw_execute(p);
	fftw_destroy_plan(p);
	ComplexD A = ComplexD(0,1/(wavelength*Z)) * expi(K*Z);
	//ComplexD A = ComplexD(0,1/(wavelength));
	//ComplexD A = expi(K*Z);
	for( int x=0; x<W; x++ )
	for( int y=0; y<W; y++ ) {
		c_fft_out[x+W*y] = A * c_fft_out[x+W*y] / W / W / W / W;
	}
	output.swapquarters();
	fftw_free(fft_f1);
	//fftw_free(fft_f2);
	//fftw_free(fft_func);
	mon.setjob(1.0);
}

bool HSpaceFourier::getconfig(csrutils::DOMSObject& st) {
	st[L"depth"] = depth;
	st[L"wavelength"] = wavelength;
	return true;
}

bool HSpaceFourier::setconfig(csrutils::DOMSObject& st) {
	depth = st[L"depth"];
	wavelength = st[L"wavelength"];
	return true;
}
