
#include "stdafx.h"
#include "image.h"
#include <csrmath_complex.h>
#include "resource.h"

using namespace csrutils;
using csrmath::ComplexD;
using csrmath::expi;

#define OFFSET(x,y,width,height)	( 3 * (x) + STRIDELENGTH(width) * ((height)-(y)-1) )
#define LIMIT(x,xmin,xmax)			( (x)<(xmin) ? (xmin) : ( (x)>(xmax) ? (xmax) : (x) ) )

/*inline void SETCOLOR(IMAGE *image, int x, int y, unsigned int color) {
	size_t offset = OFFSET(x,y,image->width,image->height);
	if( offset < STRIDELENGTH(image->width)*image->height - 4 ) {
		unsigned int *ref = (unsigned int*) ( image->data + OFFSET(x,y,image->width,image->height) );
		*ref = ( (*ref) & 0xFF000000 ) | ( color & 0x00FFFFFF );
	}
}*/

inline void SETCOLOR(Image *image, int x, int y, unsigned int color) {
	if( x < 0 || x >= (int)image->getWidth() || y < 0 || y >= (int)image->getHeight() )
		return;
	size_t offset = OFFSET(x,y,image->getWidth(),image->getHeight());
	if( offset < STRIDELENGTH(image->getWidth())*image->getHeight() - 4 ) {
		unsigned int *ref = (unsigned int*) ( image->getData() + offset );
		*ref = ( (*ref) & 0xFF000000 ) | ( color & 0x00FFFFFF );
	}
}

unsigned int HuygensColor_AbsLin(double ampl, double phase) {
	unsigned int col;
	unsigned char *rgb = (unsigned char*) &col;
	double abs = 256.0 * ampl;
	abs = LIMIT(abs,0.0,255.0);
	rgb[0] = (unsigned char) ( abs );
	rgb[1] = (unsigned char) ( abs );
	rgb[2] = (unsigned char) ( abs );
	rgb[3] = 0;
	return col;
}

unsigned int HuygensColor_TrgLin(double ampl, double phase) {
	unsigned int col;
	unsigned char *rgb = (unsigned char*) &col;
	double abs = 256.0 * ampl;
	abs = LIMIT(abs,0.0,255.0);
	phase = phase + M_PI;
	double R, G, B;
	if( phase < 2*M_PI/3 ) {
		R = 1.0 - phase / (2*M_PI/3);
		G = 0.0 + phase / (2*M_PI/3);
		B = 0.0;
		abs /= sqrt( R*R + G*G );
	} else if( phase < 4*M_PI/3 ) {
		R = 0.0;
		G = 1.0 - ( phase / (2*M_PI/3) - 1.0 );
		B = 0.0 + ( phase / (2*M_PI/3) - 1.0 );
		abs /= sqrt( G*G + B*B );
	} else {
		R = 0.0 + ( phase / (2*M_PI/3) - 2.0 );
		G = 0.0;
		B = 1.0 - ( phase / (2*M_PI/3) - 2.0 );
		abs /= sqrt( R*R + B*B );
	}
	rgb[0] = (unsigned char) ( LIMIT(abs*B,0,255) );
	rgb[1] = (unsigned char) ( LIMIT(abs*G,0,255) );
	rgb[2] = (unsigned char) ( LIMIT(abs*R,0,255) );
	rgb[3] = 0;
	return col;
}

unsigned int HuygensColor_HSL(double ampl, double phase) {
	unsigned int col;
	unsigned char *rgb = (unsigned char*) &col;
	double L = LIMIT(ampl,0.0,1.0);
	double H = LIMIT(0.5+phase / (2.0*M_PI),0.0,1.0);
	double S = 1.0;
	double Q;
	if( L < 0.5 )
		Q = L * ( 1.0 + S );
	else
		Q = L + S - L*S;
	double P = 2.0*L - Q;
	double T_R = H + 1.0/3.0;
	double T_G = H;
	double T_B = H - 1.0/3.0;
	if( T_R<0.0 ) T_R += 1.0;
	if( T_G<0.0 ) T_G += 1.0;
	if( T_B<0.0 ) T_B += 1.0;
	if( T_R>1.0 ) T_R -= 1.0;
	if( T_G>1.0 ) T_G -= 1.0;
	if( T_B>1.0 ) T_B -= 1.0;
	double R, G, B;
	if( T_R < 1.0/6.0 )
		R = P + ( (Q-P) * 6.0 * T_R );
	else if( T_R < 0.5 )
		R = Q;
	else if( T_R < 2.0/3.0 )
		R = P + ( (Q-P) * 6.0 * (2.0/3.0-T_R) );
	else
		R = P;
	if( T_G < 1.0/6.0 )
		G = P + ( (Q-P) * 6.0 * T_G );
	else if( T_G < 0.5 )
		G = Q;
	else if( T_G < 2.0/3.0 )
		G = P + ( (Q-P) * 6.0 * (2.0/3.0-T_G) );
	else
		G = P;
	if( T_B < 1.0/6.0 )
		B = P + ( (Q-P) * 6.0 * T_B );
	else if( T_B < 0.5 )
		B = Q;
	else if( T_B < 2.0/3.0 )
		B = P + ( (Q-P) * 6.0 * (2.0/3.0-T_B) );
	else
		B = P;
	rgb[0] = (unsigned char) ( LIMIT((int)(256.0*B),0,255) );
	rgb[1] = (unsigned char) ( LIMIT((int)(256.0*G),0,255) );
	rgb[2] = (unsigned char) ( LIMIT((int)(256.0*R),0,255) );
	rgb[3] = 0;
	return col;
}

void Huygens_RenderLightMap(HLightMap *map, csrutils::Image *img, double amplif, HRenderType type) {
	uint32_t x, y;
	ComplexD* data = (ComplexD*) map->data;
	for( x=0; x<map->width;  x++ )
	for( y=0; y<map->height; y++ ) {
		ComplexD c = data[ x  +map->width*y ];
		double ampl  = c.abs2() * amplif;
		double phase = c.arg();
		unsigned int col = 0;
		switch(type) {
		case HRT_AMPL:
			col = HuygensColor_AbsLin(ampl,phase);
			break;
		case HRT_COLPHASE:
			//col = HuygensColor_TrgLin(ampl,phase);
			col = HuygensColor_HSL(ampl,phase);
			break;
		}
		SETCOLOR(img,x,y,col);
	}
}