#define _HUYGENS_INT_
#define _CRT_NON_CONFORMING_SWPRINTFS
#define _CRT_SECURE_NO_WARNINGS
#include "stdafx.h"
#include "imageview_int.h"
#include "filedlg.h"
#include "resource.h"

using namespace csrutils;

const wchar_t *ImageView_ClassName = L"CLS_HIMAGEVIEW";
bool ImageView_RegClass_State = false;
std::map<HWND,HImageView*> ImageView_Wnds;
std::map<HWND,HImageView_ControlItem*> ImageViewContol_Wnds;

const FILDEDLG_FILTERSPEC ImageView_SaveTypes[] =
{
    { L"Bitmap (*.bmp)",	L"*.bmp" },
    //{ L"PNG (*.png)",		L"*.png" },
    //{ L"JPEG (*.jpg)",		L"*.jpg" },
	{ 0, 0 }
};

HImageView::HImageView(HInstance *_inst, HLightMap *_map, HWND hWndParent, bool asChild)
		: inst(_inst), map(_map) {
	ctx = new HImageView_Ctx;
	ctx->controls.inited = false;
	ctx->rendtype = HRT_AMPL;
	ctx->aschild  = asChild;
	ctx->showline = false;
	ctx->viewlineheight = 150;
	ctx->viewlineY0 = 0;
	amplif = 0.5;
	ImageView_RegClass(inst->getHInst());
	DWORD wflags = asChild ? WS_CHILD : WS_OVERLAPPEDWINDOW;
	ctx->hWnd = CreateWindow(ImageView_ClassName,L"ImageView",wflags,
								0,0,600,600,hWndParent,0,inst->getHInst(),this);
	HDC hdc = GetDC(ctx->hWnd);
	if( map!=NULL )
		ctx->image = new Image(map->width,map->height,csrutils::ImageBitFormat_RGB24,hdc,ctx->bitmap);
	else
		ctx->image = NULL;
	ctx->ihdc = CreateCompatibleDC(hdc);
	if( map!=NULL )
		SelectObject(ctx->ihdc,ctx->bitmap);
	ShowWindow(ctx->hWnd,SW_NORMAL);
    UpdateWindow(ctx->hWnd);
}

HImageView::HImageView(HInstance *_inst, HWND hWndParent, bool asChild)
		: HImageView(_inst,NULL,hWndParent,asChild) { }

HImageView::~HImageView() {
	DestroyWindow(ctx->hWnd);
	DeleteObject(ctx->ihdc);
	delete ctx->image;
	delete ctx;
}

HWND HImageView::getwnd() {
	return ctx->hWnd;
}

void ImageView_RegClass(HINSTANCE hInst) {
	if(ImageView_RegClass_State)
		return;
	ImageView_RegClass_State = true;
	WNDCLASSEX wcex;
    wcex.cbSize = sizeof(WNDCLASSEX);
    wcex.style          = CS_HREDRAW | CS_VREDRAW;
    wcex.lpfnWndProc    = ImageView_WndProc;
    wcex.cbClsExtra     = 0;
	wcex.cbWndExtra     = 0;
    wcex.hInstance      = hInst;
    wcex.hIcon          = LoadIcon(hInst, MAKEINTRESOURCE(IDI_APPICON));
    wcex.hCursor        = LoadCursor(NULL, IDC_ARROW);
    wcex.hbrBackground  = (HBRUSH)(COLOR_WINDOW+1);
    wcex.lpszMenuName   = NULL;
    wcex.lpszClassName  = ImageView_ClassName;
    wcex.hIconSm        = LoadIcon(wcex.hInstance, MAKEINTRESOURCE(IDI_APPLICATION));
    if ( !RegisterClassEx(&wcex) ) {
        return;
    }
}

LRESULT CALLBACK ImageView_WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	HImageView* view = ImageView_Wnds[hWnd];
	if( view!=NULL ) {
		return view->_wndproc(hWnd,message,wParam,lParam);
	} else {
		switch(message) {
			case WM_CREATE: {
				CREATESTRUCT* crt = (CREATESTRUCT*) lParam;
				view = (HImageView*) crt->lpCreateParams;
				ImageView_Wnds[hWnd] = view;
				//SetTimer(hWnd,0,1000,0);
				return view->_wndproc(hWnd,message,wParam,lParam);
			}
			default:
				return DefWindowProc(hWnd,message,wParam,lParam);
		}
	}
}

LRESULT APIENTRY EditSubclassProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) { 
	//HImageView* view = ImageView_Wnds[hWnd];
	HImageView_ControlItem *item = ImageViewContol_Wnds[hWnd];
    //if (uMsg == WM_GETDLGCODE) 
    //    return DLGC_WANTALLKEYS; 
	if( message==WM_KEYDOWN && wParam==0x71 )
		return item->view->_wndproc(item->view->ctx->hWnd,message,wParam,lParam);
	return CallWindowProc(item->wpProc,hWnd,message,wParam,lParam);
} 

LRESULT HImageView::_wndproc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	//HImageView* view = ImageView_Wnds[hWnd];
	switch(message) {
	case WM_CREATE: {		
		RECT rcClient, rcWindow;
		GetClientRect(hWnd,&rcClient);
		GetWindowRect(hWnd,&rcWindow);
		MoveWindow(hWnd,0,0,map->width+rcWindow.right-rcClient.right,
			map->height+rcWindow.bottom-rcClient.bottom,true);
		break; }
	case WM_DESTROY:
		break;
	case WM_TIMER:
		//InvalidateRect(hWnd,0,0);
		break;
	case WM_PAINT: {
		PAINTSTRUCT ps;
		HDC hdc = BeginPaint(hWnd,&ps);
		_paint(hdc);
		EndPaint(hWnd,&ps);
		break; }
	case WM_KEYDOWN:
		switch(wParam) {
		case 0x1B:		// ESC
			DestroyWindow(hWnd);
			break;
		case 0x53: {	// S
			csrutils::StringB path;
			bool res = QueryFileDialogue(hWnd,FILEDLG_SAVE,path,ImageView_SaveTypes,L"bmp");
			if(res)
				ctx->image->save(path.bytes(),csrutils::ImageFileFormat_BMP);
			break; }
		case 0x4F: {	// O
			csrutils::StringB path;
			bool res = QueryFileDialogue(hWnd,FILEDLG_OPEN,path,ImageView_SaveTypes,L"bmp");
			if(res) {
				openfile( File::getcurdir().getfile(path) );
			}
			break; }
		case 0x70: {	// F1
			ctx->showline ^= 1;
			update();
			break; }
		case 0x71: {	// F2
			if( !ctx->controls.inited )
				_initcontrols();
			else {
				_hidecontrols();
				update();
			}
			break; }
		case 0x77:		// F8
			switch( ctx->rendtype ) {
			case HRT_AMPL:
				ctx->rendtype = HRT_COLPHASE;
				break;
			case HRT_COLPHASE:
				ctx->rendtype = HRT_AMPL;
				break;
			}
			update();
			break;
		case 0x78:		// F9
			update();
			break;
		}
		break;
	case WM_LBUTTONDOWN: {
		/*int x = LOWORD(lParam);
		int y = HIWORD(lParam);
		if( ctx->image!=NULL ) {
			int X = x * ctx->image->getWidth()  / ctx->imgview_width;
			int Y = y * ctx->image->getHeight() / ctx->imgview_height;
			ctx->viewlineY0 = Y;
			update();
		}*/
		break; }
	default:
		return DefWindowProc(hWnd,message,wParam,lParam);
	}
	return 0;
}

void HImageView::_initcontrols() {
	ctx->controls.inited = true;
	//wchar_t buf[64];
	HFONT hFont = CreateFont(15,0,0,0,0,0,0,0,0,0,0,0,0,L"Arial");
	//swprintf(buf,L"%lf",amplif);
	csrutils::String s_amplif = csrutils::float2strw(amplif,3,3);
	//s_amplif.copy(buf);
	ctx->controls.SetAmplText.view = this;
	ctx->controls.SetAmplText.hWnd = CreateWindow(L"STATIC",L"Ampl",WS_CHILD|SS_SIMPLE,10,10,50,15,ctx->hWnd,0,0,0);
	PostMessage(ctx->controls.SetAmplText.hWnd,WM_SETFONT,(WPARAM)hFont,TRUE);
	ShowWindow(ctx->controls.SetAmplText.hWnd,SW_NORMAL);
	ImageViewContol_Wnds[ctx->controls.SetAmplText.hWnd] = &ctx->controls.SetAmplText;
	ctx->controls.SetAmpl.view = this;
	ctx->controls.SetAmpl.hWnd = CreateWindow(L"EDIT",s_amplif.bytes(),WS_CHILD|WS_BORDER,60,10,100,15,ctx->hWnd,0,0,0);
	ctx->controls.SetAmpl.wpProc = (WNDPROC) SetWindowLongPtr(
		ctx->controls.SetAmpl.hWnd,GWLP_WNDPROC,(LONG_PTR)EditSubclassProc);
	//ctx->controls.hmap[ctx->controls.SetAmpl.hWnd] = &ctx->controls.SetAmpl;
	ImageViewContol_Wnds[ctx->controls.SetAmpl.hWnd] = &ctx->controls.SetAmpl;
	PostMessage(ctx->controls.SetAmpl.hWnd,WM_SETFONT,(WPARAM)hFont,TRUE);
	ShowWindow(ctx->controls.SetAmpl.hWnd,SW_NORMAL);
}

void HImageView::_hidecontrols() {
	ctx->controls.inited = false;
	char buf[64];
	GetWindowTextA(ctx->controls.SetAmpl.hWnd,buf,32);
	//swscanf(buf,L"%lf",&amplif);
	amplif = csrutils::str2float(buf);
	DestroyWindow(ctx->controls.SetAmpl.hWnd);
	DestroyWindow(ctx->controls.SetAmplText.hWnd);
	ImageViewContol_Wnds.erase(ctx->controls.SetAmpl.hWnd);
	ImageViewContol_Wnds.erase(ctx->controls.SetAmplText.hWnd);
	InvalidateRect(ctx->hWnd,0,0);
}

void HImageView::update() {
	if( map!=NULL ) {
		if( ctx->image->getWidth() != map->width || ctx->image->getHeight() != map->height ) {
			delete ctx->image;
			ctx->image = new Image(map->width,map->height,csrutils::ImageBitFormat_RGB24,ctx->ihdc,ctx->bitmap);
			SelectObject(ctx->ihdc,ctx->bitmap);
		}
		Huygens_RenderLightMap(map,ctx->image,amplif,ctx->rendtype);
	}
	InvalidateRect(ctx->hWnd,0,0);
}

void HImageView::_paint(HDC hdc) {
	RECT rcClient;
	GetClientRect(ctx->hWnd,&rcClient);
	int dst_width  = rcClient.right - rcClient.left;
	int dst_height = rcClient.bottom - rcClient.top;
	if(ctx->showline)
		dst_height -= ctx->viewlineheight;
	if( dst_width == ctx->image->getWidth() && dst_height == ctx->image->getHeight() )
		BitBlt( hdc, 0,0, dst_width,dst_height, ctx->ihdc, 0,0, SRCCOPY );
	else {
		/*if( dst_width * ctx->image->getHeight() > dst_height * ctx->image->getWidth() )
			dst_width = dst_height * ctx->image->getWidth() / ctx->image->getHeight();
		if( dst_height * ctx->image->getWidth() > dst_width * ctx->image->getHeight() )
			dst_height = dst_width * ctx->image->getHeight() / ctx->image->getWidth();
		SetStretchBltMode(hdc,HALFTONE);
		StretchBlt( hdc, 0,0, dst_width,dst_height, ctx->ihdc,
			0,0, ctx->image->getWidth(),ctx->image->getHeight(), SRCCOPY );*/
		Huygens_CopyWithScale(hdc,ctx->ihdc,dst_width,dst_height,ctx->image->getWidth(),ctx->image->getHeight());
		// TODO:
		//ctx->imgview_width  = dst_width;
		//ctx->imgview_height = dst_height;
	}
	if(ctx->showline) {
		int _y = ctx->viewlineY0 * ctx->imgview_height / ctx->image->getHeight();
		HPEN hpen1 = CreatePen(0,1,0xFFFFFF);
		MoveToEx(hdc,0,_y,NULL);
		LineTo(hdc,rcClient.right,_y);
		SelectObject(hdc,hpen1);
		MoveToEx(hdc,0,_y+1,NULL);
		LineTo(hdc,rcClient.right,_y+1);
		int top = rcClient.bottom - ctx->viewlineheight;
		HBRUSH bg = CreateSolidBrush(0xFFFFFF);
		SelectObject(hdc,bg);
		Rectangle(hdc,0,top,rcClient.right,rcClient.bottom);
		int W = ctx->image->getWidth();
		int H0 = ctx->viewlineheight - 1;
		int W0 = rcClient.right - rcClient.left;
		int Y0 = ctx->viewlineY0;
		HPEN hpen2 = CreatePen(0,1,0xFF);
		SelectObject(hdc,hpen2);
		MoveToEx(hdc,0,rcClient.bottom,0);
		int datalen = W;
		float* data = new float[W];
		for( int X=0; X<W; X++ ) {
			uint32_t col = 0xFF & ctx->image->getPixel(X,Y0);
			data[X] = col / 256.0f;
		}
		for( int X=0; X<datalen; X++ ) {
			int x = X*W0/datalen;
			int y = top + (H0-(int)(H0*data[X]));
			LineTo(hdc,x,y);
		}
		/*int* peaks = new int[datalen];
		int peakscnt = 0;
		int peakgap = 10;
		do {
			peakgap = peakgap * 3 / 4;
			peakscnt = 0;
			int lastmin = 0;
			int lastmax = 0;
			for( int X=peakgap; X<datalen-peakgap; X++ ) {
				if( data[X-peakgap] < data[X]-0.02 && data[X+peakgap] < data[X]-0.02 && lastmax <= lastmin ) {
					peaks[peakscnt++] = X;
					lastmax = X;
					//X += 2*peakgap-1;
				}
				if( data[X-peakgap] > data[X]+0.00 && data[X+peakgap] > data[X]+0.00 ) {
					lastmin = X;
				}
			}
		} while( peakgap * peakscnt > 3 * datalen / 8 );
		HPEN hpen3 = CreatePen(0,1,0xFF0000);
		SelectObject(hdc,hpen3);
		for( int i=0; i<peakscnt; i++ ) {
			int x = peaks[i]*W0/datalen;
			MoveToEx(hdc,x,top,NULL);
			LineTo(hdc,x,top+H0);
		}
		char b[32];
		sprintf(b,"Cnt:%d (at %d)",peakscnt,peakgap);
		TextOutA(hdc,rcClient.right-100,top,b,strlen(b));*/
		delete[] data;
	}
}

void HImageView::openfile(File file) {
	delete ctx->image;
	Image* img = NULL; // TODO: Image::load(file,csrutils::ImageFileFormat_BMP);
	ctx->image = new csrutils::Image(img->getWidth(),img->getHeight(),csrutils::ImageBitFormat_RGB24,ctx->ihdc,ctx->bitmap);
	//map = new HLightMap( img->getWidth(), img->getHeight() );
	*ctx->image = *img;
	SelectObject(ctx->ihdc,ctx->bitmap);
	InvalidateRect(ctx->hWnd,NULL,false);
}

void Huygens_CopyWithScale(HDC dst, HDC src, uint32_t dst_w, uint32_t dst_h, uint32_t src_w,  uint32_t src_h) {
	int dst_width  = dst_w;
	int dst_height = dst_h;
		if( dst_width * src_h > dst_height * src_w )
			dst_width = dst_height * src_w / src_h;
		if( dst_height * src_w > dst_width * src_h )
			dst_height = dst_width * src_h / src_w;
		SetStretchBltMode(dst,HALFTONE);
		StretchBlt( dst, 0,0, dst_width,dst_height, src,
			0,0, src_w,src_h, SRCCOPY );
		//ctx->imgview_width  = dst_width;
		//ctx->imgview_height = dst_height;
}
