#define _HUYGENS_INT_

#include "stdafx.h"
#include <csrmath_complex.h>
#include "resource.h"

using namespace csrutils;
using csrmath::min;
using csrmath::max;

HLightMap::HLightMap(const HLightMap& map)
		: width(map.width), height(map.height) {
	data = new double[2*width*height];
	memcpy(data,map.data,2*sizeof(double)*width*height);
	minX = map.minX;
	maxX = map.maxX;
	minY = map.minY;
	maxY = map.maxY;
}

HLightMap::HLightMap(uint32_t _width, uint32_t _height)
		: width(_width), height(_height) {
	data = new double[2*width*height];
	minX = -0.5;
	maxX =  0.5;
	minY = -0.5;
	maxY =  0.5;
}


HLightMap::HLightMap(uint32_t _width, uint32_t _height, 
		double _minX, double _minY, double _maxX, double _maxY)
		: width(_width), height(_height) {
	data = new double[2*width*height];
	minX = _minX;
	maxX = _maxX;
	minY = _minY;
	maxY = _maxY;
}

HLightMap::~HLightMap() {
	delete[] data;
}

void HLightMap::copy(const HLightMap& map) {
	if( width!=map.width || height!=map.height )
		throw;
	memcpy(data,map.data,2*sizeof(double)*width*height);
}

void HLightMap::copy2(const HLightMap& map) {
	if( width!=map.width || height!=map.height ) 
		setsize(map.width,map.height);
	copy(map);
}

void HLightMap::copypart(const HLightMap& map) {
	uint32_t W = min(width,map.width);
	uint32_t H = min(height,map.height);
	uint32_t X0 = (width-W)/2;
	uint32_t Y0 = (height-W)/2;
	uint32_t X1 = (map.width-W)/2;
	uint32_t Y1 = (map.height-W)/2;
	for( uint32_t x=0; x<W; x++ )
	for( uint32_t y=0; y<H; y++ ) {
		size_t off0 = 2*( (x+X0) + (y+Y0)*width );
		size_t off1 = 2*( (x+X1) + (y+Y1)*map.width );
		data[off0+0] = map.data[off1+0];
		data[off0+1] = map.data[off1+1];
	}
}

void HLightMap::setsize(uint32_t _width, uint32_t _height) {
	width  = _width;
	height = _height;
	delete[] data;
	data = new double[2*width*height];
}

void HLightMap::swapquarters() {
	if( width != height || width%2 != 0 )
		throw 0;
	double *line = new double[2*width];
	int w2 = width/2;
	int w = width;
	for( int x=0; x<w2; x++ ) {
		for( int y=0; y<w; y++ ) {
			line[2*y+0] = data[2*(x+w*y)+0];
			line[2*y+1] = data[2*(x+w*y)+1];
		}
		for( int y=0; y<w; y++ ) {
			data[2*(x+w*y)+0] = data[2*((x+w2)+w*y)+0];
			data[2*(x+w*y)+1] = data[2*((x+w2)+w*y)+1];
		}
		for( int y=0; y<w; y++ ) {
			data[2*((x+w2)+w*y)+0] = line[2*y+0];
			data[2*((x+w2)+w*y)+1] = line[2*y+1];
		}
	}
	for( int y=0; y<w2; y++ ) {
		for( int x=0; x<w; x++ ) {
			line[2*x+0] = data[2*(x+w*y)+0];
			line[2*x+1] = data[2*(x+w*y)+1];
		}
		for( int x=0; x<w; x++ ) {
			data[2*(x+w*y)+0] = data[2*(x+w*(y+w2))+0];
			data[2*(x+w*y)+1] = data[2*(x+w*(y+w2))+1];
		}
		for( int x=0; x<w; x++ ) {
			data[2*(x+w*(y+w2))+0] = line[2*x+0];
			data[2*(x+w*(y+w2))+1] = line[2*x+1];
		}
	}
	delete[] line;
}
