#include "stdafx.h"
#include "lightvideo.h"

using namespace csrutils;

struct HLightVideo_Ctx {
	HANDLE hFile;
	uint32_t width;
	uint32_t height;
	uint32_t frame;
	uint32_t nframes;
	uint32_t dataoffset;
	float* framebuffer;
	HLightMap* map;
};

struct HLightVideoWriter_Ctx {
	HANDLE hFile;
	uint32_t width;
	uint32_t height;
	uint32_t frame;
	float* framebuffer;
};

struct HLVID_HEADER {
	uint32_t width;
	uint32_t height;
	uint32_t nframes;
	uint32_t dataoffset;
};

HLightVideoWriter::HLightVideoWriter(csrutils::File file, uint32_t _width, uint32_t _height) {
	ctx = new HLightVideoWriter_Ctx;
	ctx->width  = _width;
	ctx->height = _height;
	ctx->frame  = 0;
	ctx->hFile  = CreateFile( String(file.path()).bytes(), GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL );
	ctx->framebuffer = new float[2*ctx->width*ctx->height];
	HLVID_HEADER lvh;
	lvh.width      = ctx->width;
	lvh.height     = ctx->height;
	lvh.nframes    = -1;
	lvh.dataoffset = sizeof(HLVID_HEADER);
	DWORD sz;
	WriteFile(ctx->hFile,&lvh,sizeof(HLVID_HEADER),&sz,NULL);
}

HLightVideoWriter::~HLightVideoWriter() {
	HLVID_HEADER lvh;
	lvh.width      = ctx->width;
	lvh.height     = ctx->height;
	lvh.nframes    = ctx->frame;
	lvh.dataoffset = sizeof(HLVID_HEADER);
	DWORD sz;
	SetFilePointer(ctx->hFile,0,0,FILE_BEGIN);
	WriteFile(ctx->hFile,&lvh,sizeof(HLVID_HEADER),&sz,NULL);
	CloseHandle(ctx->hFile);
	delete[] ctx->framebuffer;
	delete ctx;
}

bool HLightVideoWriter::writeframe(HLightMap* map) {
	for( uint32_t x=0; x<map->width;  x++ )
	for( uint32_t y=0; y<map->height; y++ ) {
		ctx->framebuffer[2*(x+y*ctx->width)+0] = (float) map->data[2*(x+y*map->width)+0];
		ctx->framebuffer[2*(x+y*ctx->width)+1] = (float) map->data[2*(x+y*map->width)+1];
	}
	DWORD sz0 = sizeof(float)*2*ctx->width*ctx->height;
	DWORD sz;
	WriteFile(ctx->hFile,ctx->framebuffer,sz0,&sz,NULL);
	ctx->frame ++;
	return sz==sz0;
}

HLightVideo::HLightVideo(csrutils::File file) {
	ctx = new HLightVideo_Ctx;
	ctx->hFile = CreateFile(String(file.path()).bytes(),GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
	DWORD sz;
	HLVID_HEADER lvh;
	ReadFile(ctx->hFile,&lvh,sizeof(HLVID_HEADER),&sz,NULL);
	ctx->width      = lvh.width;
	ctx->height     = lvh.height;
	ctx->dataoffset = lvh.dataoffset;
	ctx->nframes    = lvh.nframes;
	ctx->framebuffer = new float[2*ctx->width*ctx->height];
	ctx->map = new HLightMap(ctx->width,ctx->height);
	seekframe(0);
}

HLightVideo::~HLightVideo() {
	CloseHandle(ctx->hFile);
	delete ctx;
}

bool HLightVideo::getframe(HLightMap* map) {
	DWORD sz0 = sizeof(float)*2*ctx->width*ctx->height;
	DWORD sz;
	ReadFile(ctx->hFile,ctx->framebuffer,sz0,&sz,NULL);
	if( sz!=sz0 )
		return false;
	for( uint32_t x=0; x<map->width;  x++ )
	for( uint32_t y=0; y<map->height; y++ ) {
		map->data[2*(x+y*map->width)+0] = ctx->framebuffer[2*(x+y*ctx->width)+0];
		map->data[2*(x+y*map->width)+1] = ctx->framebuffer[2*(x+y*ctx->width)+1];
	}
	ctx->frame ++;
	return true;
}

uint32_t HLightVideo::getframescount() {
	return ctx->nframes;
}

uint32_t HLightVideo::getframeid() {
	return ctx->frame;
}

bool HLightVideo::nextframe() {
	if( ctx->frame >= ctx->nframes )
		return false;
	ctx->frame ++;
	return true;
}

bool HLightVideo::seekframe(uint32_t frameid) {
	if( frameid >= ctx->nframes )
		return false;
	ctx->frame = frameid;
	int offset = ctx->dataoffset + (sizeof(float)*2*ctx->width*ctx->height) * frameid;
	SetFilePointer(ctx->hFile,offset,0,FILE_BEGIN);
}

HLightMap* HLightVideo::getmap() {
	return ctx->map;
}