#define _HUYGENS_INT_
#define _USE_MATH_DEFINES
#include "stdafx.h"
#include "hoptelems.h"
#include <math.h>
#include <csrmath.h>
#include <csrmath_complex.h>

using namespace csrutils;
using csrmath::ComplexD;
using csrmath::expi;

/**************************************
 **       Class HSpaceHuygnes        **
 **************************************/

HOpticalElementClass *clsHSpaceHuygens = 0;
const wchar_t* szClsHSpaceHuygens = L"SpaceHuygens";

HOpticalElement* createHSpaceHuygens(const String& name) {
	return new HSpaceHuygens(name);
}

const HOpticalElementClass* getHSpaceHuygensClass() {
	if( clsHSpaceHuygens==0 ) {
		clsHSpaceHuygens = new HOpticalElementClass;
		clsHSpaceHuygens->nmType = szClsHSpaceHuygens;
		clsHSpaceHuygens->createElement = createHSpaceHuygens;
	}
	return clsHSpaceHuygens;
}

HSpaceHuygens::HSpaceHuygens(const String& nmTag)
		: HOpticalElement(szClsHSpaceHuygens,nmTag), depth(10.0), wavelength(0.005) {
}

HSpaceHuygens::HSpaceHuygens(const String& nmTag, double _depth, double _wavelength)
		: HOpticalElement(szClsHSpaceHuygens,nmTag), depth(_depth), wavelength(_wavelength) {
	restep = 8;
}

void HSpaceHuygens::run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon) {
	double K = 2.0*M_PI / wavelength;
	double Z = depth;
	double perstep = 1.0 / output.width;
	for( uint32_t x=0; x<output.width;  x+=restep ) {
	for( uint32_t y=0; y<output.height; y+=restep ) {
		double X = output.getX(x);
		double Y = output.getY(y);
		ComplexD S = 0.0;
		for( uint32_t x0=0; x0<input.width;  x0+=restep )
		for( uint32_t y0=0; y0<input.height; y0+=restep ) {
			double X0 = input.getX(x0);
			double Y0 = input.getY(y0);
			ComplexD p = ComplexD(input.data[2*(x0+y0*input.width)+0],input.data[2*(x0+y0*input.width)+1]);
			if( p!=ComplexD(0.0) ) {
				double r = sqrt( (X-X0)*(X-X0) + (Y-Y0)*(Y-Y0) + Z*Z );
				S += expi( r*K ) / r * p;
			}
		}
		S *= (input.maxX-input.minX)*(input.maxY-input.minY)/input.width/input.height*restep*restep;
		S /= ComplexD(0,wavelength);
		for( int dx=0; dx<restep; dx++ )
		for( int dy=0; dy<restep; dy++ ) {
			if( x+dx < output.width && y+dy < output.height ) {
				output.data[2*((x+dx)+(y+dy)*output.width)+0] = S.Re();
				output.data[2*((x+dx)+(y+dy)*output.width)+1] = S.Im();
			}
		}
		//img->updmap[x+y*img->width] = true;
		//if( ctx->monitor->check_abort() )
		//	return;
	}
		mon.setjob(x*perstep);
		if( mon.checkabort() ) {
			mon.setcompleted();
			return;
		}
	}
}

bool HSpaceHuygens::getconfig(csrutils::DOMSObject& st) {
	st[L"depth"] = depth;
	st[L"wavelength"] = wavelength;
	st[L"restep"] = restep;
	return true;
}

bool HSpaceHuygens::setconfig(csrutils::DOMSObject& st) {
	depth = st[L"depth"];
	wavelength = st[L"wavelength"];
	restep = st[L"restep"];
	return true;
}





/**************************************
 **        Class HSpaceFresnel        **
 **************************************/

HOpticalElementClass *clsHSpaceFresnel = 0;
const wchar_t* szClsHSpaceFresnel = L"SpaceFresnel";

HOpticalElement* createHSpaceFresnel(const String& name) {
	return new HSpaceFresnel(name);
}

const HOpticalElementClass* getHSpaceFresnelClass() {
	if( clsHSpaceFresnel==0 ) {
		clsHSpaceFresnel = new HOpticalElementClass;
		clsHSpaceFresnel->nmType = szClsHSpaceFresnel;
		clsHSpaceFresnel->createElement = createHSpaceFresnel;
	}
	return clsHSpaceFresnel;
}

HSpaceFresnel::HSpaceFresnel(const String& nmTag)
		: HOpticalElement(szClsHSpaceFresnel,nmTag), depth(10.0), wavelength(0.005) {
	restep = 4;
}

HSpaceFresnel::HSpaceFresnel(const String& nmTag, double _depth, double _wavelength)
		: HOpticalElement(szClsHSpaceFresnel,nmTag), depth(_depth), wavelength(_wavelength) {
	restep = 4;
}

void HSpaceFresnel::run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon) {
	double K = 2.0*M_PI / wavelength;
	double Z = depth;
	double perstep = 1.0 / output.width;
	ComplexD A = ComplexD(0,1/(wavelength*Z)) * expi(K*Z);
	double B = K / (2*Z);
	for( uint32_t x=0; x<output.width;  x+=restep ) {
	for( uint32_t y=0; y<output.height; y+=restep ) {
		double X = output.getX(x);
		double Y = output.getY(y);
		ComplexD S = 0.0;
		for( uint32_t x0=0; x0<input.width;  x0+=restep )
		for( uint32_t y0=0; y0<input.height; y0+=restep ) {
			double X0 = input.getX(x0);
			double Y0 = input.getY(y0);
			ComplexD p = ComplexD(input.data[2*(x0+y0*input.width)+0],input.data[2*(x0+y0*input.width)+1]);
			if( p!=ComplexD(0.0) ) {
				S += p * expi( B*( (X-X0)*(X-X0) + (Y-Y0)*(Y-Y0) ) );
			}
		}
		S *= A;
		S *= (input.maxX-input.minX)*(input.maxY-input.minY)/input.width/input.height*restep*restep;
		for( int dx=0; dx<restep; dx++ )
		for( int dy=0; dy<restep; dy++ ) {
			if( x+dx < output.width && y+dy < output.height ) {
				output.data[2*((x+dx)+(y+dy)*output.width)+0] = S.Re();
				output.data[2*((x+dx)+(y+dy)*output.width)+1] = S.Im();
			}
		}
		//img->updmap[x+y*img->width] = true;
		//if( ctx->monitor->check_abort() )
		//	return;
	}
		mon.setjob(x*perstep);
		if( mon.checkabort() ) {
			mon.setcompleted();
			return;
		}
	}
}

bool HSpaceFresnel::getconfig(csrutils::DOMSObject& st) {
	st[L"depth"] = depth;
	st[L"wavelength"] = wavelength;
	st[L"restep"] = restep;
	return true;
}

bool HSpaceFresnel::setconfig(csrutils::DOMSObject& st) {
	depth = st[L"depth"];
	wavelength = st[L"wavelength"];
	restep = st[L"restep"];
	return true;
}



/**************************************
 **        Class HSpaceHShift        **
 **************************************/

HOpticalElementClass *clsHSpaceShift = 0;
const wchar_t* szClsHSpaceShift = L"SpaceShift";

HOpticalElement* createHSpaceShift(const String& name) {
	return new HSpaceShift(name);
}

const HOpticalElementClass* getHSpaceShiftClass() {
	if( clsHSpaceShift==0 ) {
		clsHSpaceShift = new HOpticalElementClass;
		clsHSpaceShift->nmType = szClsHSpaceShift;
		clsHSpaceShift->createElement = createHSpaceShift;
	}
	return clsHSpaceShift;
}

HSpaceShift::HSpaceShift(const String& nmTag)
		: HOpticalElement(szClsHSpaceShift,nmTag), depth(10.0), wavelength(0.005) {
}

HSpaceShift::HSpaceShift(const String& nmTag, double _depth, double _wavelength)
		: HOpticalElement(szClsHSpaceShift,nmTag), depth(_depth), wavelength(_wavelength) {
}

void HSpaceShift::run(HLightMap& input, HLightMap& output, csrutils::ThreadMonitor &mon) {
	double K = 2.0*M_PI / wavelength;
	ComplexD P = expi( depth*K );
	for( uint32_t x=0; x<output.width; x++ )
	for( uint32_t y=0; y<output.width; y++ ) {
		ComplexD p = ComplexD(input.data[2*(x+y*input.width)+0],input.data[2*(x+y*input.width)+1]);
		p *= P;
		output.data[2*(x+y*output.width)+0] = p.Re();
		output.data[2*(x+y*output.width)+1] = p.Im();
	}
}

bool HSpaceShift::getconfig(csrutils::DOMSObject& st) {
	st[L"depth"] = depth;
	st[L"wavelength"] = wavelength;
	return true;
}

bool HSpaceShift::setconfig(csrutils::DOMSObject& st) {
	depth = st[L"depth"];
	wavelength = st[L"wavelength"];
	return true;
}
