#include "stdafx.h"

#include "tvid.h"

using namespace csrutils;

#define TVID_WIDTH		320
#define TVID_HEIGHT		240
#define TVID_HEADER_SZ	0xFF

#define STRIDELENGTH(width)	((((width*3-1)/4)+1)*4)
#define OFFSET(x,y,width,height) ( 3 * (x) + STRIDELENGTH(width) * ((height)-(y)-1) )
#define LIMIT(x,xmin,xmax)	( (x)<(xmin) ? (xmin) : ( (x)>(xmax) ? (xmax) : (x) ) )

struct HTVIDVideo_Ctx {
	HANDLE hFile;
	float* framedata;
	uint32_t frameid;
	HLightMap* map;
};

HTVIDVideo::HTVIDVideo(csrutils::File file) {
	ctx = new HTVIDVideo_Ctx;
	ctx->hFile = CreateFile(String(file.path()).bytes(),GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
	ctx->framedata = new float[TVID_WIDTH*TVID_HEIGHT];
	ctx->map = new HLightMap(TVID_WIDTH,TVID_HEIGHT);
	seekframe(0);
}

HTVIDVideo::~HTVIDVideo() {
	CloseHandle(ctx->hFile);
	delete[] ctx->framedata;
	delete ctx;
}

bool HTVIDVideo::getframe(HLightMap* map) {
	int x, y;
	for( x=0; x<TVID_WIDTH; x++ )
	for( y=0; y<TVID_HEIGHT; y++ ) {
		map->data[2*(x+y*map->width)+0] = ctx->framedata[x+y*TVID_WIDTH];
		map->data[2*(x+y*map->width)+1] = 0.0;
	}
	return true;
}

uint32_t HTVIDVideo::getframescount() {
	uint64_t filesize;
	GetFileSizeEx(ctx->hFile,(PLARGE_INTEGER)&filesize);
	uint32_t nframes = (uint32_t)( (filesize-TVID_HEADER_SZ) / (TVID_WIDTH*TVID_HEIGHT*4) );
	return nframes;
}

uint32_t HTVIDVideo::getframeid() {
	return ctx->frameid;
}

bool HTVIDVideo::seekframe(uint32_t frameid) {
	ctx->frameid = frameid;
	int offset = TVID_HEADER_SZ + (TVID_WIDTH*TVID_HEIGHT*4) * frameid;
	SetFilePointer(ctx->hFile,offset,0,FILE_BEGIN);
	DWORD sz;
	ReadFile(ctx->hFile,ctx->framedata,TVID_WIDTH*TVID_HEIGHT*4,&sz,0);
	return true;
}

bool HTVIDVideo::nextframe() {
	DWORD sz;
	ctx->frameid++;
	ReadFile(ctx->hFile,ctx->framedata,TVID_WIDTH*TVID_HEIGHT*4,&sz,0);
	return true;
}

HLightMap* HTVIDVideo::getmap() {
	return ctx->map;
}

/*bool TVID_LoadFrame(IMAGE *img, const char *path, int frameid) {
	HANDLE hFile = CreateFileA(path,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
	int offset = 0xFF + (TVID_WIDTH*TVID_HEIGHT*4) * frameid;
	SetFilePointer(hFile,offset,0,FILE_BEGIN);
	float *frame = new float[TVID_WIDTH*TVID_HEIGHT];
	DWORD sz;
	ReadFile(hFile,frame,TVID_WIDTH*TVID_HEIGHT*4,&sz,0);
	CloseHandle(hFile);
	int x, y;
	for( x=0; x<TVID_WIDTH; x++ )
	for( y=0; y<TVID_HEIGHT; y++ ) {
		int off = OFFSET(x,y,img->width,img->height);
		float val = 2.0f * frame[x+y*TVID_WIDTH];
		unsigned char c1 = (unsigned char) LIMIT((int)val,0x00,0xFF);
		img->data[off+0] = c1;
		img->data[off+1] = c1;
		img->data[off+2] = c1;
	}
	delete[] frame;
	return true;
}*/
