#define _HUYGENS_INT_
#define _CRT_NON_CONFORMING_SWPRINTFS
#define _CRT_SECURE_NO_WARNINGS
#include "stdafx.h"
#include "filedlg.h"
#include "resource.h"

using namespace csrutils;

struct HVideoView_Ctx {
	HWND hWnd;
	HImageView* imageview;
	HLightMap* framemap;
	HWND hImgWnd;
	bool aschild;
	int playbarhight;
	bool mousedown;
};

LRESULT CALLBACK VideoView_WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

const wchar_t *VideoView_ClassName = L"CLS_HVideoView";
bool VideoView_RegClass_State = false;
std::map<HWND,HVideoView*> VideoView_Wnds;
//std::map<HWND,HVideoView_ControlItem*> VideoViewContol_Wnds;

const FILDEDLG_FILTERSPEC VideoView_Types[] =
{
    { L"TVID (*.tvid)",   L"*.tvid" },
    //{ L"PNG (*.png)",   L"*.png" },
    //{ L"JPEG (*.jpg)",   L"*.jpg" },
	{ 0, 0 }
};

void VideoView_RegClass(HINSTANCE hInst) {
	if(VideoView_RegClass_State)
		return;
	VideoView_RegClass_State = true;
	WNDCLASSEX wcex;
    wcex.cbSize = sizeof(WNDCLASSEX);
    wcex.style          = CS_HREDRAW | CS_VREDRAW;
    wcex.lpfnWndProc    = VideoView_WndProc;
    wcex.cbClsExtra     = 0;
	wcex.cbWndExtra     = 0;
    wcex.hInstance      = hInst;
    wcex.hIcon          = LoadIcon(hInst, MAKEINTRESOURCE(IDI_APPICON));
    wcex.hCursor        = LoadCursor(NULL, IDC_ARROW);
    wcex.hbrBackground  = (HBRUSH)(COLOR_WINDOW+1);
    wcex.lpszMenuName   = NULL;
    wcex.lpszClassName  = VideoView_ClassName;
    wcex.hIconSm        = LoadIcon(wcex.hInstance, MAKEINTRESOURCE(IDI_APPLICATION));
    if ( !RegisterClassEx(&wcex) ) {
        return;
    }
}

LRESULT CALLBACK VideoView_WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	HVideoView* view = VideoView_Wnds[hWnd];
	if( view!=NULL ) {
		return view->_wndproc(hWnd,message,wParam,lParam);
	} else {
		switch(message) {
			case WM_CREATE: {
				CREATESTRUCT* crt = (CREATESTRUCT*) lParam;
				view = (HVideoView*) crt->lpCreateParams;
				VideoView_Wnds[hWnd] = view;
				return view->_wndproc(hWnd,message,wParam,lParam);
			}
			default:
				return DefWindowProc(hWnd,message,wParam,lParam);
		}
	}
}

HVideoView::HVideoView(HInstance* _inst, HVideo* _video, HWND hWndParent, bool asChild)
		: inst(_inst), video(_video) {
	ctx = new HVideoView_Ctx;
	VideoView_RegClass(inst->getHInst());
	DWORD wflags = asChild ? WS_CHILD : WS_OVERLAPPEDWINDOW;
	ctx->hWnd = CreateWindow(VideoView_ClassName,L"VideoView",wflags,
								0,0,600,600,hWndParent,0,inst->getHInst(),this);
	ctx->framemap = video->getmap();
	ctx->playbarhight = 50;
	ctx->imageview = new HImageView(inst,ctx->framemap,ctx->hWnd,true);
	ctx->hImgWnd = ctx->imageview->getwnd();
	ShowWindow(ctx->hWnd,SW_NORMAL);
    UpdateWindow(ctx->hWnd);
}

LRESULT HVideoView::_wndproc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	//HImageView* view = ImageView_Wnds[hWnd];
	switch(message) {
	case WM_CREATE: {		
		RECT rcClient, rcWindow;
		GetClientRect(hWnd,&rcClient);
		GetWindowRect(hWnd,&rcWindow);
		SetTimer(hWnd,0,30,NULL);
		break; }
	case WM_DESTROY:
		break;
	case WM_TIMER: {
		video->getframe(ctx->framemap);
		ctx->imageview->update();
		video->nextframe();
		RECT rcClient;
		GetClientRect(hWnd,&rcClient);
		rcClient.top = rcClient.bottom-ctx->playbarhight;
		InvalidateRect(hWnd,&rcClient,false);
		break; }
	case WM_SIZE: {
		RECT rcClient;
		GetClientRect(hWnd,&rcClient);
		MoveWindow(ctx->hImgWnd,0,0,rcClient.right,rcClient.bottom-ctx->playbarhight,false);
		break; }
	case WM_PAINT: {
		PAINTSTRUCT ps;
		HDC hdc = BeginPaint(hWnd,&ps);
		_paint(hdc);
		EndPaint(hWnd,&ps);
		break; }
	case WM_KEYDOWN:
		switch(wParam) {
		case 0x1B:		// ESC
			DestroyWindow(hWnd);
			break;
		case 0x53: {	// S
			/*csrutils::StringW path;
			bool res = QueryFileDialogue(hWnd,FILEDLG_SAVE,path,ImageView_SaveTypes,L"bmp");
			if(res)
				ctx->image->save(path.bytes(),csrutils::ImageFileFormat_BMP);*/
			break; }
		case 0x4F: {	// O
			/*csrutils::StringW path;
			bool res = QueryFileDialogue(hWnd,FILEDLG_OPEN,path,ImageView_SaveTypes,L"bmp");
			if(res) {
				openfile(File(path));
			}*/
			break; }
		case 0x70:	// F1
		case 0x71:	// F2
		case 0x77:	// F8
			SendMessage(ctx->hImgWnd,WM_KEYDOWN,wParam,lParam);
			break;
		}
		break;
	case WM_LBUTTONDOWN:
		ctx->mousedown = true;
		break;
	case WM_LBUTTONUP: {
		int x = LOWORD(lParam);
		int y = HIWORD(lParam);
		ctx->mousedown = false;
		RECT rcClient;
		GetClientRect(ctx->hWnd,&rcClient);
		_vidseek( (float) x / rcClient.right );
		break; }
	default:
		return DefWindowProc(hWnd,message,wParam,lParam);
	}
	return 0;
}

void HVideoView::_paint(HDC hdc) {
	HPEN pen1 = CreatePen(0,1,0x808080);
	HBRUSH brush1 = CreateSolidBrush(0xE0E0E0);
	SelectObject(hdc,pen1);
	SelectObject(hdc,brush1);
	RECT rcClient;
	GetClientRect(ctx->hWnd,&rcClient);
	int Y0 = rcClient.bottom-ctx->playbarhight;
	int H0 = ctx->playbarhight;
	int W0 = rcClient.right;
	Rectangle(hdc,0,Y0,W0,Y0+H0);
	Rectangle(hdc,0,Y0+13,W0,Y0+18);
	float p = (float) video->getframeid() / video->getframescount();
	int x = (int)( W0*p );
	Rectangle(hdc,x-5,Y0+7,x+5,Y0+23);
}

void HVideoView::_vidseek(float p) {
	int frame = (int)( video->getframescount() * p );
	video->seekframe(frame);
}